# Copyright (C) 2013 Red Hat, Inc.  All rights reserved.
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
#
# Authors: Jan Safranek <jsafrane@redhat.com>
# -*- coding: utf-8 -*-
"""
    Basic infrastructure for asynchronous jobs. All necessary CIM classes and
    indications are implemented here.

    .. autoclass:: JobManager
        :members:

    .. autoclass:: Job
        :members:

    .. autoclass:: LMI_ConcreteJob
        :members:

    .. autoclass:: LMI_OwningJobElement
        :members:

    .. autoclass:: LMI_AffectedJobElement
        :members:

    .. autoclass:: LMI_MethodResult
        :members:

    .. autoclass:: LMI_AssociatedJobMethodResult
        :members:
"""

from datetime import datetime, timedelta
import threading
from Queue import Queue
import pywbem
from pywbem.cim_provider2 import CIMProvider2
import socket
import traceback

from lmi.providers import cmpi_logging, parse_instance_id
from lmi.providers.IndicationManager import IndicationManager

LOG = cmpi_logging.get_logger(__name__)

@cmpi_logging.trace_function
def register_filters(job_clsname, indication_manager=None):
    """
    This function registers static indication filters at IndicationManager.
    It should be called upon provider's initialization.

    :param job_clsname: (``String``) CIM class name for asynchonous jobs.
        Will be part of filter queries.
    :param indication_manager: If not given, global instance will be obtained.
    """
    if indication_manager is None:
        ind_manager = IndicationManager.get_instance()
    filters = {}
    query_args = {
            "classname" : job_clsname,
            "prefix"    : indication_manager.nameprefix
        }
    for fltr_id, fltr_props in JobManager.IND_FILTERS.items():
        filters[fltr_id] = fltr_props.copy()
        filters[fltr_id]['Query'] = fltr_props['Query'] % query_args
    indication_manager.add_filters(job_clsname, filters)

# Too many instance attributes
# pylint: disable-msg=R0902
class Job(object):
    """
        Generic abstract class representing one CIM_ConcreteJob.
        It remembers input and output arguments, affected ManagedElements and
        owning ManagedElement (to be able to create associations to them)
        and all CIM_ConcreteJob properties.

        Due to multiple threads processing the job, each job has its own
        lock to guard its status changes. It is expected that number of jobs
        is quite low.
    """

    DEFAULT_TIME_BEFORE_REMOVAL = 60  # in seconds

    STATE_QUEUED = 1  # Job has not started yet
    STATE_RUNNING = 2  # Job is running
    STATE_FINISHED_OK = 3  # Job finished OK
    STATE_FAILED = 4  # Job finished with error
    STATE_SUSPENDED = 5  # Job is queued and suspended
    STATE_TERMINATED = 6  # Job was queued and terminated

    FINAL_STATES = [STATE_FINISHED_OK, STATE_FAILED, STATE_SUSPENDED,
            STATE_TERMINATED]

    # There is no way how to suspend/terminate running job!

    @cmpi_logging.trace_method
    def __init__(self, job_manager, job_name, input_arguments,
            method_name, affected_elements, owning_element):
        """
        Create new storage job.

        :param job_manager: (``JobManager``) Reference to ``JobManager``, which
            will manage this job.
        :param job_name: (``string``) User-friendly name of the job.
        :param input_arguments: (``dictionary param_name -> param_value``)
            Input arguments of the method, which spawned this job.
        :param method_name: (``string``) Name of the CIM method, which spawned
            this job.
        :param affected_elements: (``array of CIMInstanceName``) List of
            affected elements.  ``LMI_AffectedJobElement`` association will be
            created for them.
        :param owning_element: (``CIMInstanceName``) Reference to service, which
            spawned the job. ``LMI_OwningJobElement`` association will be
            created for it.
        """
        self.job_manager = job_manager
        self.timer_manager = job_manager.timer_manager

        # Unique ID
        self.the_id = job_manager.get_next_id()

        # User friendly name of the job
        self.job_name = job_name

        # Dictionary of input arguments, 'parameter_name' -> 'parameter_value'
        # The parameter value must be CIMProperty or something that can be
        # assigned to it.
        self.input_arguments = input_arguments

        # Dictionary of output arguments, 'parameter_name' -> 'parameter_value'
        # The parameter value must be CIMProperty or something that can be
        # assigned to it.
        self.output_arguments = None

        # Method return value, as CIMProperty or something that can be
        # assigned to it.
        self.return_value = None
        # Value of Job.ReturnValueType
        self.return_value_type = None

        # Name of the method
        self.method_name = method_name

        # Time when the job was created
        self.time_submitted = datetime.utcnow()

        # Nr. of seconds before the job is removed when the job finishes
        self.time_before_removal = self.DEFAULT_TIME_BEFORE_REMOVAL

        # If the job should be removed after completion
        self.delete_on_completion = True

        self.percent_complete = 0

        # State of the job
        self.job_state = self.STATE_QUEUED

        # Last change of job state, wall clock time
        self.clocktime_of_last_state_change = self.time_submitted

        # Duration of the job in RUNNING state (in seconds)
        self.elapsed_time = None

        # When the job started (= switched to RUNNING), wall clock time
        self.start_clocktime = None
        # When the job started (= switched to RUNNING), monotonic clock time
        self.start_monotime = None
        # When the job finished (= switched from RUNNING), monotonic clock time
        self.finish_monotime = None

        # Array of CIMInstanceNames of affected elements, so we can
        # enumerate associations to them.
        self.affected_elements = affected_elements

        # CIMInstanceName to owning element (service), so we can enumerate
        # instances.
        self.owning_element = owning_element

        # Timer used to delete the job after time_before_removal seconds
        self.timer = None

        # CIMError with result code
        self.error = None

        # internal lock to protect state changes from races
        self._lock = threading.RLock()

        self._execute = None
        self._execargs = None
        self._execkwargs = None
        self._cancel = None
        self._cancelargs = None
        self._cancelkwargs = None

        self._finished_event = threading.Event()

    @cmpi_logging.trace_method
    def set_execute_action(self, callback, *args, **kwargs):
        """
        Set callback, which will be called when the job is to be executed. It is
        expected that the callback will take some time to execute. The callback
        must change state of the job and set output parameters and error in a
        thread-safe way, i.e. by calling ``finish_method()``.

        :param callback: (``function``) Reference to callback to call.
        :param args, kwargs: All other parameters will be passed to the
            callback. It is highly recommended to add reference to the job
            to the callback.
        """
        self._execute = callback
        self._execargs = args
        self._execkwargs = kwargs

    @cmpi_logging.trace_method
    def set_cancel_action(self, callback, *args, **kwargs):
        """
        Set callbacks, which will be called when the job is to be
        cancelled. The callback must be quick, the job is already locked!

        :param callback: (``function``) Reference to callback to call.
        :param args, kwargs: All other parameters will be passed to the
            callback. It is highly recommended to add reference to the job
            to the callback.
        """
        self._cancel = callback
        self._cancelargs = args
        self._cancelkwargs = kwargs

    @cmpi_logging.trace_method
    def finish_method(self, new_state, return_value=None, return_type=None,
            output_arguments=None, error=None, affected_elements=None):
        """
        Mark the job as finished, with given return value, output parameters and
        error.
        This method is thread-safe.

        :param new_state: (``Job.STATE_* value``) Resulting state of the job.
        :param return_value: (``string``) Return value of the job, encoded
            into string. Can be None when the job does not return any value.
        :param return_type: (``Job.RetunValueType.* value``) Type of the return
            value. Can be None when the job does not return any value.
        :param output_arguments: (``dictionary param_name -> param_value``)
            Output arguments of the job. Can be None when the job does not have
            any output parameters.
        :param error: (``CIMError``) Error raised by the job. Can be None,
            when the job finished successfully.
        :param affected_elements: (``array of CIMInstanceName``) New list of
            affected elements to generate LMI_<name>JobAffectedElement
            association. If None, the old list, passed to constructor, remains
            untouched.
        """
        self.lock()
        self.return_value = return_value
        self.return_value_type = return_type
        self.output_arguments = output_arguments
        self.error = error
        if affected_elements is not None:
            self.affected_elements = affected_elements
        self.change_state(new_state, 100)
        self.unlock()
        self._finished_event.set()

    @cmpi_logging.trace_method
    def change_state(self, new_state, percent=None):
        """
        Change state of a job. (Re-)calculate various times based on the state
        change. Send indications as necessary.
        This method is thread-safe.

        :param new_state: (``Job.STATE_* value``) New state of the job.
            It can be the same as the previous state to indicate progress of
            the job.
        :param percent: (``int``)) Percent complete of the job. When None,
            this valuu will be automatically calculated (in progress = 50%,
            finished = 100%).
        """
        self.lock()

        LOG().debug("Job %s: %s changes state from %d to %d",
                self.the_id, self.job_name, self.job_state, new_state)

        # For sending indications
        prev_instance = None
        send_indication = False
        indication_ids = []

        if self.job_state != new_state:
            # Remember to send indications
            prev_instance = self.job_manager.get_job_instance(self)
            send_indication = True
            indication_ids.append(JobManager.IND_JOB_CHANGED)

            # Check if the job has just finished
            if (self.job_state not in self.FINAL_STATES
                    and new_state in self.FINAL_STATES):
                # Remember finish time
                self.finish_clocktime = datetime.utcnow()
                self.finish_monotime = self.timer_manager.now()
                # Remember job execution time.
                if self.start_monotime:
                    self.elapsed_time = self.finish_monotime \
                        - self.start_monotime
                # Send indication
                if new_state == self.STATE_FAILED:
                    indication_ids.append(JobManager.IND_JOB_FAILED)
                if new_state == self.STATE_FINISHED_OK:
                    indication_ids.append(JobManager.IND_JOB_SUCCEEDED)

            # Check if the job has just started
            if new_state == self.STATE_RUNNING:
                self.start_clocktime = datetime.utcnow()
                self.start_monotime = self.timer_manager.now()

            self.clocktime_of_last_state_change = datetime.utcnow()
            self.job_state = new_state

        if percent is None:
            # guess the percentage from status
            if new_state == self.STATE_QUEUED:
                percent = 0
            elif new_state == self.STATE_RUNNING:
                percent = 50
            else:
                percent = 100
        if self.percent_complete != percent:
            # Remember to send indications
            if not send_indication:
                self.clocktime_of_last_state_change = datetime.utcnow()
                prev_instance = self.job_manager.get_job_instance(self)
                send_indication = True
            indication_ids.append(JobManager.IND_JOB_PERCENT_UPDATED)
            self.percent_complete = percent

        if send_indication:
            current_instance = self.job_manager.get_job_instance(self)
            self.job_manager.send_modify_indications(
                    prev_instance, current_instance, indication_ids)

        # start / update the timer if necesasry
        self._restart_timer()
        self.unlock()

    @cmpi_logging.trace_method
    def _expire(self):
        """
        Callback when a Job completes and time_before_removal second passed.
        The job gets removed from its JobManager.
        """
        LOG().debug("Job %s: %s expired", self.the_id, self.job_name)

        self.job_manager.remove_job(self)

    @cmpi_logging.trace_method
    def _restart_timer(self):
        """
        Re-schedule timer for TimeBeforeRemoval because some property has
        changed.
        """
        if not self.job_state in self.FINAL_STATES:
            return

        # Stop the old timer.
        if self.timer:
            self.timer.cancel()
            self.timer = None

        # Start the new timer.
        if self.delete_on_completion:
            now = self.timer_manager.now()
            passed = now - self.finish_monotime
            timeout = self.time_before_removal - passed
            if timeout <= 0:
                # Just in case...
                self._expire()
                return

            LOG().debug("Starting timer for job %s: '%s' for %f seconds",
                    self.the_id, self.job_name, timeout)
            self.timer = self.timer_manager.create_timer(
                    "Job " + self.job_name,
                    callback=self._expire)
            self.timer.start(timeout)

    @cmpi_logging.trace_method
    def lock(self):
        """
        Lock internal mutex. Other threads will block on subsequent lock().
        The lock is recursive, i.e. can be called multiple times from
        single thread.
        """
        self._lock.acquire()

    @cmpi_logging.trace_method
    def unlock(self):
        """ Unlock internal mutex."""
        self._lock.release()

    @cmpi_logging.trace_method
    def execute(self):
        """
        Start executing the job. It calls the execute callback, set by
        ``set_execute_action()``.

        job_state must be already set to STATE_RUNNING.
        Any exception is translated to CIMError and appropriate state is set.
        """
        try:
            self._execute(*(self._execargs), **(self._execkwargs))
        except pywbem.CIMError, error:
            LOG().trace_warn("Job.execute caught an CIMError %s", str(error))
            LOG().trace_info("traceback: %s", traceback.format_exc())
            self.finish_method(Job.STATE_FAILED, error=error)
        except Exception, ex:
            LOG().trace_warn("Job.execute caught an Exception %s", str(ex))
            LOG().trace_info("traceback: %s", traceback.format_exc())
            error = pywbem.CIMError(pywbem.CIM_ERR_FAILED, str(ex))
            self.finish_method(Job.STATE_FAILED, error=error)

    @cmpi_logging.trace_method
    def cancel(self):
        """
        Cancels queued action. The action must have not been started.
        """
        self.change_state(self.STATE_TERMINATED)
        if self._cancel:
            self._cancel(*(self._cancelargs), **(self._cancelkwargs))
        self._finished_event.set()

    @cmpi_logging.trace_method
    def get_name(self):
        """
        Return CIMInstanceName of the job.

        :rtype: ``CIMInstanceName``
        """
        name = pywbem.CIMInstanceName(
                classname=self.job_manager.job_classname,
                namespace=self.job_manager.namespace,
                keybindings={
                        'InstanceID': self.get_instance_id()
        })
        return name

    @cmpi_logging.trace_method
    def get_instance_id(self, classname=None):
        """
        Return InstanceID.

        :param classname: (``string``) Optional classname to generate InstanceID
            for different class, e.g. for LMI_<name>MethodResult.
        :rtype: ``string``
        """
        if classname is None:
            classname = self.job_manager.job_classname
        return 'LMI:' + classname + ':' + str(self.the_id)

    @cmpi_logging.trace_method
    def get_pre_call(self):
        """
        Return indication that describes the pre-execution values of the
        job's invocation.

        :rtype: ``CIMInstance of CIM_InstMethodCall``
        """
        path = pywbem.CIMInstanceName(
                classname="CIM_InstMethodCall",
                keybindings={},
                host=socket.gethostname(),
                namespace=self.job_manager.namespace)
        inst = pywbem.CIMInstance(
                classname="CIM_InstMethodCall",
                path=path)
        src_instance = self._get_cim_instance()
        inst['SourceInstance'] = src_instance
        inst['SourceInstanceModelPath'] = str(src_instance.path)
        inst['MethodName'] = self.method_name
        # TODO: uncomment when Pegasus can correctly handle instances
        # of unregistered classes
        # inst['MethodParameters'] = self.get_method_params(
        #        '__MethodParameters', True, False)
        inst['PreCall'] = True
        return inst

    @cmpi_logging.trace_method
    def get_cim_error(self):
        """
        Return job error as CIMInstance of CIM_Error.
        :returns: CIMInstance of CIM_Error
        """
        path = pywbem.CIMInstanceName(
                classname="CIM_Error",
                host=socket.gethostname(),
                namespace=self.job_manager.namespace)
        err = pywbem.CIMInstance(
                classname="CIM_Error",
                path=path)
        err['CIMStatusCode'] = pywbem.Uint32(self.error[0])
        err['Message'] = self.error[1]
        return err

    @cmpi_logging.trace_method
    def get_post_call(self):
        """
        Return indication that describes the post-execution values of the
        job's invocation.

        :rtype: ``CIMInstance of CIM_InstMethodCall``
        """
        path = pywbem.CIMInstanceName(
                classname="CIM_InstMethodCall",
                keybindings={},
                host=socket.gethostname(),
                namespace=self.job_manager.namespace)
        inst = pywbem.CIMInstance(
                classname="CIM_InstMethodCall",
                path=path)

        src_instance = self._get_cim_instance()
        inst['SourceInstance'] = src_instance
        inst['SourceInstanceModelPath'] = str(src_instance.path)
        inst['MethodName'] = self.method_name
        # TODO: add input parameters too when Pegasus can correctly handle
        # instances of unregistered classes
        # TODO: also fix the class name to __MethodParameters
        params = self.get_method_params(
                '__MethodParameters_' + self.method_name, False, True)
        if params:
            inst['MethodParameters'] = params
        inst['PreCall'] = False

        if self.return_value_type is not None:
            inst['ReturnValueType'] = self.return_value_type
        if self.return_value is not None:
            inst['ReturnValue'] = str(self.return_value)
        if self.error is not None:
            err = self.get_cim_error()
            inst['Error'] = [err, ]
        return inst

    @cmpi_logging.trace_method
    def _get_cim_instance(self):
        """
        Return CIMInstance of this job.

        :rtype: CIMInstance
        """
        return self.job_manager.get_job_instance(self)

    @cmpi_logging.trace_method
    def get_method_params(self, class_name, include_input, include_output):
        """
        Create a class of given name with all input or output parameters
        of the asynchronous method. Typically used to assemble
        CIM_ConcreteJob.JobInParameters or CIM_InstMethodCall.MethodParameters
        values.

        :param class_name: (``string``) Name of the class to create.
        :param input: (``boolean``) Whether input parameters should be
            included in the returned class
        :param output: (``boolean``) Whether output parameters should be
            included in the returned class
        :rtype: CIMInstance of the created class.
        """
        path = pywbem.CIMInstanceName(
                classname=class_name,
                namespace=self.job_manager.namespace)
        inst = pywbem.CIMInstance(classname=class_name, path=path)
        if include_input and self.input_arguments:
            for (name, value) in self.input_arguments.iteritems():
                inst[name] = value
        if include_output and self.output_arguments:
            # overwrite any input parameter
            for (name, value) in self.output_arguments.iteritems():
                inst[name] = value
        return inst


    @cmpi_logging.trace_method
    def wait_for_job(self, timeout=None):
        """
        Block and wait until the job completes.

        :param timeout: (``float``) Number of seconds to wait for the job
            to complete.
        :rtype: ``bool`` - True, when the job is finished, False if the timeout
            occurred.
        """
        return self._finished_event.wait(timeout)

    # pylint: disable-msg=R0903
    class ReturnValueType(object):
        """ CIM_InstMethodCall.ReturnValueType values."""
        Boolean = pywbem.Uint16(2)
        String = pywbem.Uint16(3)
        Char16 = pywbem.Uint16(4)
        Uint8 = pywbem.Uint16(5)
        Sint8 = pywbem.Uint16(6)
        Uint16 = pywbem.Uint16(7)
        Sint16 = pywbem.Uint16(8)
        Uint32 = pywbem.Uint16(9)
        Sint32 = pywbem.Uint16(10)
        Uint64 = pywbem.Uint16(11)
        Sint64 = pywbem.Uint16(12)
        Datetime = pywbem.Uint16(13)
        Real32 = pywbem.Uint16(14)
        Real64 = pywbem.Uint16(15)
        Reference = pywbem.Uint16(16)

class JobManager(object):
    """
    Container of all queued, running or finished ``LMI_ConcreteJobs``.

    Usage:

     1. Create MOF file for these classes:

        * ``LMI_<name>Job``

        * ``LMI_<name>MethodResult``

        * ``LMI_Affected<name>JobElement``

        * ``LMI_Owning<name>JobElement``

        * ``LMI_Associated<name>JobMethodResult``

        Where ``<name>`` is prefix of your classes, for example 'Storage'

     2. During initialization, initialize ``TimerManager`` and create
        ``JobManager``.

     3. When needed. create new Job instance:

     4. Set its execute callback using ``set_execute_action()``. This callback
        will be called when the job is to be executed. It will be called in
        context of ``JobManager`` worker thread!

     5. Optionally, set cancel callback using ``set_execute_action()``. This
        callback will be called when the job is still queued and is cancelled by
        application. This callback will be called in context of CIMOM callback
        and should be quick!

     6. Enqueue the job using ``JobManager.add_job()`` method.

     7. When your execute callback is called, you can optionally call
        ``job.change_state()`` to update percentage of completion.

     8. When your execute callback is finished, don't forget to set method
        result using ``job.finish_method()``.

    * ``JobManager`` automatically sends all job-related indications.
    * ``Job`` automatically tracks various timestamps.
    * By default, the job automatically disappears after 60 seconds after it
      finishes. Application may set ``DeleteOnCompletion`` and
      ``TimeBeforeRemoval`` properties of ``LMI_<name>Job`` to override this
      timeout.
    """

    COMMAND_STOP = 1

    IND_JOB_PERCENT_UPDATED = "PercentUpdated"
    IND_JOB_SUCCEEDED = "Succeeded"
    IND_JOB_FAILED = "Failed"
    IND_JOB_CHANGED = "Changed"
    IND_JOB_CREATED = "Created"

    IND_FILTERS = {
        IND_JOB_PERCENT_UPDATED: {
            "Query" : "SELECT * FROM LMI_%(prefix)sInstModification WHERE "
                "SourceInstance ISA %(classname)s AND "
                "SourceInstance.CIM_ConcreteJob::PercentComplete <> "
                "PreviousInstance.CIM_ConcreteJob::PercentComplete",
            "Description" : "Modification of Percentage Complete for a "
                "Concrete Job.",
        },
        IND_JOB_SUCCEEDED: {
            "Query" : "SELECT * FROM LMI_%(prefix)sInstModification WHERE "
                "SourceInstance ISA %(classname)s AND "
                "SourceInstance.CIM_ConcreteJob::JobState = 17",
            "Description": "Modification of Job State for a "
                "Concrete Job to 'Complete'.",
        },
        IND_JOB_FAILED: {
            "Query" : "SELECT * FROM LMI_%(prefix)sInstModification WHERE "
                "SourceInstance ISA %(classname)s AND "
                "SourceInstance.CIM_ConcreteJob::JobState = 10",
            "Description": "Modification of Job State for a "
                "Concrete Job to 'Exception'.",
        },
        IND_JOB_CHANGED: {
            "Query" : "SELECT * FROM LMI_%(prefix)sInstModification WHERE "
                "SourceInstance ISA %(classname)s AND "
                "SourceInstance.CIM_ConcreteJob::JobState <> "
                "PreviousInstance.CIM_ConcreteJob::JobState",
            "Description": "Modification of Job State for a ConcreteJob.",
        },
        IND_JOB_CREATED: {
            "Query" : "SELECT * FROM LMI_%(prefix)sInstCreation WHERE "
                "SourceInstance ISA %(classname)s",
            "Description": "Creation of a ConcreteJob.",
        },
    }

    @cmpi_logging.trace_method
    def __init__(self, name, namespace, indication_manager, timer_manager):
        """
        Initialize new Manager. It automatically registers all job-related
        filters to indication_manager and starts a worker thread.

        :param name: (``string``) String with classname infix. For example
            'Storage' for ``LMI_StorageJob``, ``LMI_StorageJobMethodResult``
            etc.
        :param namespace: (``string``) Namespace of all providers.
        :param indication_manager: (``IndicationManager``): a manager where
            indications and filters should be added.
        :param timer_manager: (``TimerManager``): Timer manager instance.
        """
        # List of all jobs. Dictionary job_id -> Job.
        self.jobs = {}
        # Queue of jobs scheduled to execute.
        self.queue = Queue()
        # Last created job_id.
        self.last_instance_id = 0
        # Classname infix.
        self.name = name
        # CIMProvider2 instances for job classes.
        self.providers = {}
        self.namespace = namespace
        self.indication_manager = indication_manager
        self.timer_manager = timer_manager

        # Start the worker thread (don't forget to register it at CIMOM)
        self.worker = threading.Thread(target=self._worker_main)
        self.worker.daemon = False
        self.worker.start()

        # Various classnames for job-related classes, with correct infixes.
        self.job_classname = 'LMI_' + self.name + 'Job'
        self.method_result_classname = "LMI_" + self.name + "MethodResult"
        self.affected_classname = "LMI_Affected" + self.name + "JobElement"
        self.owning_classname = "LMI_Owning" + self.name + "JobElement"
        self.associated_result_classname = ('LMI_Associated' + self.name
                + 'JobMethodResult')
        self.indication_filter_classname = ('LMI_' + self.name
                + 'JobIndicationFilter')
        self.job_provider = None
        self._add_indication_filters()

    @cmpi_logging.trace_method
    def _add_indication_filters(self):
        """
        Add all job-related ``IndicationFilters`` to indication manager.
        """
        register_filters(self.job_classname, self.indication_manager)

    @cmpi_logging.trace_method
    def get_providers(self):
        """
        Get dictionary of providers for these classes:

        * ``LMI_<name>Job``
        * ``LMI_<name>MethodResult``
        * ``LMI_Affected<name>JobElement``
        * ``LMI_Owning<name>JobElement``
        * ``LMI_Associated<name>JobMethodResult``

        :rtype: dictionary class_name -> CIMProvider2
        """

        if not self.providers:
            job_provider = LMI_ConcreteJob(self.job_classname, job_manager=self)
            self.providers[self.job_classname] = job_provider
            self.job_provider = job_provider

            provider = LMI_MethodResult(
                    self.method_result_classname, job_manager=self)
            self.providers[self.method_result_classname] = provider

            provider = LMI_AffectedJobElement(
                    self.affected_classname, job_manager=self)
            self.providers[self.affected_classname] = provider

            provider = LMI_OwningJobElement(
                    self.owning_classname, job_manager=self)
            self.providers[self.owning_classname] = provider

            provider = LMI_AssociatedJobMethodResult(
                    self.owning_classname, job_manager=self)
            self.providers[self.associated_result_classname] = provider

        return self.providers

    @cmpi_logging.trace_method
    def add_job(self, job):
        """
        Enqueue new job. Send indication when needed.

        :param job: (``Job``) A job to enqueue.
        """
        LOG().debug("Job %s: '%s' enqueued", job.the_id, job.job_name)

        self.jobs[job.the_id] = job
        self.queue.put(job)
        # send indication
        if self.indication_manager.is_subscribed(
                        self.job_classname, self.IND_JOB_CREATED):
            job_instance = self.get_job_instance(job)
            self.indication_manager.send_instcreation(
                    job_instance, self.IND_JOB_CREATED)

    def send_modify_indications(self, prev_instance, current_instance,
            indication_ids):
        """
        Send InstModification. This is helper method called by ``Job`` when
        needed.

        :param prev_instance: Instance of ``LMI_<name>Job`` before it was
            modified.
        :param current_instance: Instance of ``LMI_<name>Job`` after it was
            modified.
        """
        for _id in indication_ids:
            self.indication_manager.send_instmodification(prev_instance,
                    current_instance, _id)

    @cmpi_logging.trace_method
    def remove_job(self, job):
        """
        Remove existing job. Note that jobs are removed automatically after a
        timeout, providers should not call this method directly.

        :param job: (``Job``) Job to remove.
        """
        LOG().debug("Job %s: '%s' removed from queue.",
                job.the_id, job.job_name)
        del self.jobs[job.the_id]
        # The job may still be in the queue!
        # There is no way, how to remove it, it will be skipped by the
        # worker thread.

    @cmpi_logging.trace_method
    def get_job_for_instance_id(self, instance_id, classname=None):
        """
        Return Job for given InstanceID or None when no such Job exist.

        :param instance_id: (``string``) InstanceID value to parse.
        :param classname: (``string``) Optional classname to parse the
            InstanceID (e.g. when parsing InstanceID of
            ``LMI_<name>MethodResult``).
        :rtype: ``Job``
        """
        if classname is None:
            classname = self.job_classname
        the_id = parse_instance_id(instance_id, classname)
        if not the_id.isdigit():
            return None
        return self.jobs.get(the_id, None)

    @cmpi_logging.trace_method
    def _worker_main(self):
        """
        This is the main loop of the job queue. It just processes enqueued
        jobs and never ends.
        """
        LOG().info("Started Job thread.")
        while True:
            command = self.queue.get()
            if isinstance(command, Job):
                # we need to protect from changes between checking state and
                # setting new state
                job = command
                job.lock()
                if job.job_state == Job.STATE_QUEUED:
                    # the job was not cancelled
                    job.change_state(Job.STATE_RUNNING)
                    job.unlock()
                    LOG().info("Starting job %s: '%s'",
                            job.the_id, job.job_name)

                    job.execute()
                    if job.error:
                        LOG().warn("Job %s: '%s' finished with error: %s",
                                job.the_id, job.job_name, str(job.error))
                    else:
                        LOG().info("Job %s: '%s' finished OK",
                                job.the_id, job.job_name)
                else:
                    # just skip suspended and terminated jobs
                    job.unlock()

            elif isinstance(command, int):
                self.queue.task_done()
                break

            self.queue.task_done()

        LOG().info("Stopped Job thread.")

    @cmpi_logging.trace_method
    def get_next_id(self):
        """
        Return next unused job id.

        :rtype: string
        """
        self.last_instance_id += 1
        return str(self.last_instance_id)

    @cmpi_logging.trace_method
    def get_job_instance(self, job):
        """
        Return CIMInstance for given job.

        :param job: (``Job``)
        :rtype: ``CIMInstance``
        """
        path = pywbem.CIMInstanceName(
                classname=self.job_classname,
                keybindings={'InstanceID': job.get_instance_id()},
                host=socket.gethostname(),
                namespace=self.namespace)
        inst = pywbem.CIMInstance(classname=self.job_classname, path=path)
        inst['InstanceID'] = job.get_instance_id()
        return self.job_provider.get_instance(None, inst)

    @cmpi_logging.trace_method
    def shutdown(self, timeout=1):
        """
        Stop the thread. If a job is running, it may leave the job process
        (mkfs, resize2fs, ...) and the worker thread (waiting for the process to
        finish) still running.

        JobManager still needs Indication Manager and TimeManager working at
        this point!

        :param timeout: Nr. of seconds to wait for the current job. Afterwards
            the thread is abandoned, leaving the process still running.
        """
        # Empty the queue, we don't want the worker to proceed with any other
        # queued job.
        while not self.queue.empty():
            queue.get(False)
            queue.task_done()

        self.queue.put(self.COMMAND_STOP)
        self.worker.join(timeout)

        # Cancel all running/suspended/queued jobs.
        # This will send indications.
        for job in self.jobs.itervalues():
            if job.job_state in (Job.STATE_QUEUED, Job.STATE_SUSPENDED,
                    Job.STATE_RUNNING):
                job.cancel()

        if self.worker.isAlive():
            # There is no way, how to stop the thread in Python, so abandon it.
            self.worker.daemon = True
            self.indication_manager = None
            self.timer_manager = None

    def can_shutdown(self):
        """
        Return True, if there is no running Job.
        """
        return self.queue.empty()


class LMI_ConcreteJob(CIMProvider2):
    """
    Provider of LMI_ConcreteJob class or its subclass.
    """
    @cmpi_logging.trace_method
    def __init__(self, classname, job_manager):
        self.classname = classname
        self.job_manager = job_manager

    @cmpi_logging.trace_method
    def enum_instances(self, env, model, keys_only):
        """
        Provider implementation of EnumerateInstances intrinsic method.
        """
        model.path.update({'InstanceID': None})
        for job in self.job_manager.jobs.values():
            model['InstanceID'] = job.get_instance_id()
            if keys_only:
                yield model
            else:
                yield self.get_instance(env, model, job)

    @cmpi_logging.trace_method
    def get_job_states(self, job):
        """
        Return JobState and OperationalStatus property values.

        :param job: (``int``) Job.STATE_* value.
        :rtype: tuple ``(JobState, OperationalStatus)`` values.
        """
        if job.job_state == Job.STATE_QUEUED:
            jobstate = self.Values.JobState.New
            opstate = [self.Values.OperationalStatus.Dormant]
        elif job.job_state == Job.STATE_RUNNING:
            jobstate = self.Values.JobState.Running
            opstate = [self.Values.OperationalStatus.OK]
        elif job.job_state == Job.STATE_FINISHED_OK:
            jobstate = self.Values.JobState.Completed
            opstate = [self.Values.OperationalStatus.OK,
                    self.Values.OperationalStatus.Completed]
        elif job.job_state == Job.STATE_SUSPENDED:
            jobstate = self.Values.JobState.Suspended
            opstate = [self.Values.OperationalStatus.OK]
        elif job.job_state == Job.STATE_FAILED:
            jobstate = self.Values.JobState.Exception
            opstate = [self.Values.OperationalStatus.Error,
                    self.Values.OperationalStatus.Completed]
        elif job.job_state == Job.STATE_TERMINATED:
            jobstate = self.Values.JobState.Terminated
            opstate = [self.Values.OperationalStatus.Stopped]
        return jobstate, opstate

    @cmpi_logging.trace_method
    # pylint: disable-msg=W0221
    def get_instance(self, env, model, job=None):
        """
        Provider implementation of GetInstance intrinsic method.
        """
        if not job:
            instance_id = model['InstanceID']
            job = self.job_manager.get_job_for_instance_id(instance_id)
        if not job:
            raise pywbem.CIMError(pywbem.CIM_ERR_NOT_FOUND,
                    "Job not found.")

        model['DeleteOnCompletion'] = job.delete_on_completion
        model['Name'] = job.job_name

        # convert seconds to timedelta
        seconds = job.time_before_removal
        if seconds:
            delta = timedelta(seconds=seconds)
            model['TimeBeforeRemoval'] = pywbem.CIMDateTime(delta)
        else:
            model['TimeBeforeRemoval'] = pywbem.CIMProperty(
                    name='TimeBeforeRemoval',
                    value=None,
                    type='datetime')

        if job.clocktime_of_last_state_change:
            model['TimeOfLastStateChange'] = pywbem.CIMDateTime(
                    job.clocktime_of_last_state_change)
        else:
            model['TimeOfLastStateChange'] = pywbem.CIMProperty(
                    name='TimeOfLastStateChange',
                    value=None,
                    type='datetime')

        if job.elapsed_time:
            elapsed_time = timedelta(seconds=job.elapsed_time)
            model['ElapsedTime'] = pywbem.CIMDateTime(elapsed_time)
        else:
            model['ElapsedTime'] = pywbem.CIMProperty(
                    name='ElapsedTime',
                    value=None,
                    type='datetime')

        model['Description'] = job.job_name
        model['LocalOrUtcTime'] = self.Values.LocalOrUtcTime.UTC_Time
        model['PercentComplete'] = pywbem.Uint16(job.percent_complete)
        if job.start_clocktime:
            model['StartTime'] = pywbem.CIMDateTime(job.start_clocktime)
        else:
            model['StartTime'] = pywbem.CIMProperty(
                    name='StartTime',
                    value=None,
                    type='datetime')

        # TODO: uncomment when Pegasus can correctly handle instances
        # of unregistered classes
        # if job.input_arguments:
        #    model['JobInParameters'] = job.get_method_params(
        #            "__JobInParameters", True, False)

        if job.job_state in Job.FINAL_STATES:
            # assemble output parameters with return value
            # TODO: use __JobOutParameters when Pegasus can create instances
            # of unregistered classes
            outparams = job.get_method_params(
                    "__MethodParameters_" + job.method_name + "_Result",
                    False,
                    True)
            if job.return_value is not None:
                outparams['__ReturnValue'] = job.return_value
            model['JobOutParameters'] = outparams

        model['TimeSubmitted'] = pywbem.CIMDateTime(job.time_submitted)
        # set correct state
        jobstate, opstate = self.get_job_states(job)
        model['JobState'] = jobstate
        model['OperationalStatus'] = opstate
        return model

    @cmpi_logging.trace_method
    def set_instance(self, env, instance, modify_existing):
        """Return a newly created or modified instance.

        :param env: Provider Environment (pycimmb.ProviderEnvironment)
        :param instance: The new pywbem.CIMInstance.  If modifying an existing
            instance, the properties on this instance have been filtered by
            the PropertyList from the request.
        :param modify_existing: True if ModifyInstance, False if CreateInstance

        Return the new instance.  The keys must be set on the new instance.
        """
        if not modify_existing:
            raise pywbem.CIMError(pywbem.CIM_ERR_NOT_SUPPORTED,
                    "Creation of Job instances is not supported.")

        job = self.job_manager.get_job_for_instance_id(instance['InstanceID'])
        if not job:
            raise pywbem.CIMError(pywbem.CIM_ERR_NOT_FOUND,
                    "Job not found.")

        try:
            job.lock()
            restart_timer = False

            for (key, value) in instance.iteritems():
                if value is None:
                    continue
                if key == 'DeleteOnCompletion':
                    job.delete_on_completion = value
                    restart_timer = True
                elif key == 'TimeBeforeRemoval':
                    job.time_before_removal = value.total_seconds()
                    restart_timer = True
                elif key == 'JobRunTimes':
                    if value != 1:
                        raise pywbem.CIMError(pywbem.CIM_ERR_NOT_SUPPORTED,
                                "JobRunTimes property is not supported.")
                elif key == 'LocalOrUtcTime':
                    if value != self.Values.LocalOrUtcTime.UTC_Time:
                        raise pywbem.CIMError(pywbem.CIM_ERR_NOT_SUPPORTED,
                                "Setting of LocalOrUtcTime property is not"
                                " supported.")
                else:
                    raise pywbem.CIMError(pywbem.CIM_ERR_NOT_SUPPORTED,
                            "Setting of %s property is not supported." % (key,))

            if restart_timer:
                job._restart_timer()
        finally:
            job.unlock()
        return instance

    @cmpi_logging.trace_method
    def delete_instance(self, env, instance_name):
        """Delete an instance.

        :param env: Provider Environment (pycimmb.ProviderEnvironment)
        :param instance_name: A pywbem.CIMInstanceName specifying the instance
            to delete.
        """
        job = self.job_manager.get_job_for_instance_id(
                instance_name['InstanceID'])
        if not job:
            raise pywbem.CIMError(pywbem.CIM_ERR_NOT_FOUND,
                    "Job not found.")
        if not job.job_status in Job.FINAL_STATES:
            raise pywbem.CIMError(pywbem.CIM_ERR_FAILED,
                    "Job has not finished.")

        self.job_manager.remove_job(job)

    @cmpi_logging.trace_method
    def cim_method_geterrors(self, env, object_name):
        """Implements LMI_StorageJob.GetErrors()

        If JobState is "Completed" and Operational Status is "Completed"
        then no instance of CIM_Error is returned.

        If JobState is "Exception" then GetErrors may return intances of
        CIM_Error related to the execution of the procedure or method invoked by
        the job.

        If Operatational Status is not "OK" or "Completed" then
        GetErrors may return CIM_Error instances related to the running of
        the job.

        :param env: -- Provider Environment (pycimmb.ProviderEnvironment)
        :param object_name: -- A pywbem.CIMInstanceName or pywbem.CIMCLassName
            specifying the object on which the method GetErrors()
            should be invoked.

        Output parameters:

        * Errors -- (type pywbem.CIMInstance(classname='CIM_Error', ...))
            If the OperationalStatus on the Job is not "OK", then this
            method will return one or more CIM Error instance(s).
            Otherwise, when the Job is "OK", null is returned.
        """
        job = self.job_manager.get_job_for_instance_id(
                object_name['InstanceID'])
        if not job:
            raise pywbem.CIMError(pywbem.CIM_ERR_NOT_FOUND,
                    "Job not found.")

        if job.error is None:
            errors = []
        else:
            err = job.get_cim_error()
            errors = [err, ]
        out_params = [
                pywbem.CIMParameter(
                        name='Errors',
                        value=errors,
                        type='instance',
                        is_array=True,
                        array_size=len(errors))
        ]
        rval = self.Values.GetErrors.Success

        return (rval, out_params)

    @cmpi_logging.trace_method
    def cim_method_requeststatechange(self, env, object_name,
                                      param_requestedstate=None,
                                      param_timeoutperiod=None):
        """Implements LMI_StorageJob.RequestStateChange()

        Requests that the state of the job be changed to the value
        specified in the RequestedState parameter. Invoking the
        RequestStateChange method multiple times could result in earlier
        requests being overwritten or lost.

        If 0 is returned, then the
        task completed successfully. Any other return code indicates an
        error condition.

        :param env: Provider Environment (pycimmb.ProviderEnvironment)
        :param object_name:  A pywbem.CIMInstanceName or pywbem.CIMCLassName
            specifying the object on which the method RequestStateChange()
            should be invoked.
        :param param_requestedstate:  The input parameter RequestedState (type pywbem.Uint16 self.Values.RequestStateChange.RequestedState)
            RequestStateChange changes the state of a job. The possible
            values are as follows: Start (2) changes the state to
            \'Running\'. Suspend (3) stops the job temporarily. The
            intention is to subsequently restart the job with \'Start\'.
            It might be possible to enter the \'Service\' state while
            suspended. (This is job-specific.) Terminate (4) stops the
            job cleanly, saving data, preserving the state, and shutting
            down all underlying processes in an orderly manner. Kill (5)
            terminates the job immediately with no requirement to save
            data or preserve the state. Service (6) puts the job into a
            vendor-specific service state. It might be possible to restart
            the job.

        :param param_timeoutperiod: --  The input parameter TimeoutPeriod (type pywbem.CIMDateTime)
            A timeout period that specifies the maximum amount of time that
            the client expects the transition to the new state to take.
            The interval format must be used to specify the TimeoutPeriod.
            A value of 0 or a null parameter indicates that the client has
            no time requirements for the transition. If this property
            does not contain 0 or null and the implementation does not
            support this parameter, a return code of \'Use Of Timeout
            Parameter Not Supported\' must be returned.
        """
        job = self.job_manager.get_job_for_instance_id(
                object_name['InstanceID'])
        if not job:
            raise pywbem.CIMError(pywbem.CIM_ERR_NOT_FOUND,
                    "Job not found.")

        try:
            job.lock()
            states = self.Values.RequestStateChange.RequestedState
            retcodes = self.Values.RequestStateChange
            if param_requestedstate == states.Suspend:
                if job.job_state != Job.STATE_QUEUED:
                    # Can suspend only queued jobs
                    rval = retcodes.Invalid_State_Transition
                else:
                    job.change_state(Job.STATE_SUSPENDED)
                    rval = retcodes.Completed_with_No_Error

            elif param_requestedstate == states.Terminate:
                if job.job_state not in (Job.STATE_QUEUED, Job.STATE_SUSPENDED):
                    # Can terminate only queued or suspended jobs
                    rval = retcodes.Invalid_State_Transition
                else:
                    job.cancel()
                    rval = retcodes.Completed_with_No_Error

            elif param_requestedstate == states.Start:
                if job.job_state != Job.STATE_SUSPENDED:
                    # Can start only suspended jobs
                    rval = retcodes.Invalid_State_Transition
                else:
                    job.change_state(Job.STATE_QUEUED)
                    # Enqueue the job again, it may be already processed
                    # (we might get the job in the queue twice, but
                    # we have only one worker thread so it won't collide).
                    self.job_manager.add_job(job)
                    rval = retcodes.Completed_with_No_Error

            else:
                rval = retcodes.Invalid_State_Transition
        finally:
            job.unlock()
        return (rval, [])

    @cmpi_logging.trace_method
    def cim_method_killjob(self, env, object_name,
                           param_deleteonkill=None):
        """Implements LMI_StorageJob.KillJob() """
        raise pywbem.CIMError(pywbem.CIM_ERR_NOT_SUPPORTED)

    @cmpi_logging.trace_method
    def cim_method_geterror(self, env, object_name):
        """Implements LMI_StorageJob.GetError()

        GetError is deprecated because Error should be an array,not a
        scalar.

        When the job is executing or has terminated without
        error, then this method returns no CIM_Error instance. However, if
        the job has failed because of some internal problem or because the
        job has been terminated by a client, then a CIM_Error instance is
        returned.

        :param env: Provider Environment (pycimmb.ProviderEnvironment)
        :param object_name: A pywbem.CIMInstanceName or pywbem.CIMCLassName
            specifying the object on which the method GetError()
            should be invoked.

        Output parameters:

        * Error -- (``pywbem.CIMInstance(classname='CIM_Error', ...)``)
          If the OperationalStatus on the Job is not "OK", then this
          method will return a CIM Error instance. Otherwise, when the
          Job is "OK", null is returned.
        """
        job = self.job_manager.get_job_for_instance_id(
                object_name['InstanceID'])
        if not job:
            raise pywbem.CIMError(pywbem.CIM_ERR_NOT_FOUND,
                    "Job not found.")

        if job.error is None:
            error = pywbem.CIMParameter(
                        name='error',
                        value=None,
                        type='instance',
                        is_array=False)
        else:
            err = job.get_cim_error()
            error = pywbem.CIMParameter(
                        name='error',
                        value=err,
                        type='instance')
        rval = self.Values.GetError.Success
        return (rval, [error])

    class Values(object):
        class JobState(object):
            New = pywbem.Uint16(2)
            Starting = pywbem.Uint16(3)
            Running = pywbem.Uint16(4)
            Suspended = pywbem.Uint16(5)
            Shutting_Down = pywbem.Uint16(6)
            Completed = pywbem.Uint16(7)
            Terminated = pywbem.Uint16(8)
            Killed = pywbem.Uint16(9)
            Exception = pywbem.Uint16(10)
            Service = pywbem.Uint16(11)
            Query_Pending = pywbem.Uint16(12)
            # DMTF_Reserved = 13..32767
            # Vendor_Reserved = 32768..65535

        class LocalOrUtcTime(object):
            Local_Time = pywbem.Uint16(1)
            UTC_Time = pywbem.Uint16(2)

        class OperationalStatus(object):
            Unknown = pywbem.Uint16(0)
            Other = pywbem.Uint16(1)
            OK = pywbem.Uint16(2)
            Degraded = pywbem.Uint16(3)
            Stressed = pywbem.Uint16(4)
            Predictive_Failure = pywbem.Uint16(5)
            Error = pywbem.Uint16(6)
            Non_Recoverable_Error = pywbem.Uint16(7)
            Starting = pywbem.Uint16(8)
            Stopping = pywbem.Uint16(9)
            Stopped = pywbem.Uint16(10)
            In_Service = pywbem.Uint16(11)
            No_Contact = pywbem.Uint16(12)
            Lost_Communication = pywbem.Uint16(13)
            Aborted = pywbem.Uint16(14)
            Dormant = pywbem.Uint16(15)
            Supporting_Entity_in_Error = pywbem.Uint16(16)
            Completed = pywbem.Uint16(17)
            Power_Mode = pywbem.Uint16(18)
            Relocating = pywbem.Uint16(19)
            # DMTF_Reserved = ..
            # Vendor_Reserved = 0x8000..

        class GetErrors(object):
            Success = pywbem.Uint32(0)
            Not_Supported = pywbem.Uint32(1)
            Unspecified_Error = pywbem.Uint32(2)
            Timeout = pywbem.Uint32(3)
            Failed = pywbem.Uint32(4)
            Invalid_Parameter = pywbem.Uint32(5)
            Access_Denied = pywbem.Uint32(6)
            # DMTF_Reserved = ..
            # Vendor_Specific = 32768..65535

        class GetError(object):
            Success = pywbem.Uint32(0)
            Not_Supported = pywbem.Uint32(1)
            Unspecified_Error = pywbem.Uint32(2)
            Timeout = pywbem.Uint32(3)
            Failed = pywbem.Uint32(4)
            Invalid_Parameter = pywbem.Uint32(5)
            Access_Denied = pywbem.Uint32(6)
            # DMTF_Reserved = ..
            # Vendor_Specific = 32768..65535

        class RequestStateChange(object):
            Completed_with_No_Error = pywbem.Uint32(0)
            Not_Supported = pywbem.Uint32(1)
            Unknown_Unspecified_Error = pywbem.Uint32(2)
            Can_NOT_complete_within_Timeout_Period = pywbem.Uint32(3)
            Failed = pywbem.Uint32(4)
            Invalid_Parameter = pywbem.Uint32(5)
            In_Use = pywbem.Uint32(6)
            # DMTF_Reserved = ..
            Method_Parameters_Checked___Transition_Started = pywbem.Uint32(4096)
            Invalid_State_Transition = pywbem.Uint32(4097)
            Use_of_Timeout_Parameter_Not_Supported = pywbem.Uint32(4098)
            Busy = pywbem.Uint32(4099)
            # Method_Reserved = 4100..32767
            # Vendor_Specific = 32768..65535
            class RequestedState(object):
                Start = pywbem.Uint16(2)
                Suspend = pywbem.Uint16(3)
                Terminate = pywbem.Uint16(4)
                Kill = pywbem.Uint16(5)
                Service = pywbem.Uint16(6)
                # DMTF_Reserved = 7..32767
                # Vendor_Reserved = 32768..65535

class LMI_OwningJobElement(CIMProvider2):
    """ Instrumentation of LMI_OwningJobElement class and its subclasses."""

    @cmpi_logging.trace_method
    def __init__(self, classname, job_manager):
        self.classname = classname
        self.job_manager = job_manager

    @cmpi_logging.trace_method
    def get_instance(self, env, model):
        """Return an instance."""
        instance_id = model['OwnedElement']['InstanceID']
        job = self.job_manager.get_job_for_instance_id(instance_id)
        if not job:
            raise pywbem.CIMError(pywbem.CIM_ERR_NOT_FOUND,
                    "OwnedElement not found.")

        if job.owning_element != model['OwningElement']:
            raise pywbem.CIMError(pywbem.CIM_ERR_NOT_FOUND,
                    "OwnedElement is not associated to OwningElement.")
        return model

    @cmpi_logging.trace_method
    def enum_instances(self, env, model, keys_only):
        """Enumerate instances."""
        model.path.update({'OwnedElement': None, 'OwningElement': None})
        for job in self.job_manager.jobs.values():
            if job.owning_element:
                model['OwnedElement'] = job.get_name()
                model['OwningElement'] = job.owning_element
                yield model

    @cmpi_logging.trace_method
    def references(self, env, object_name, model, result_class_name, role,
                   result_role, keys_only):
        """Instrument Associations."""
        ch = env.get_cimom_handle()
        if ch.is_subclass(object_name.namespace,
                  sub=object_name.classname,
                  super='CIM_ManagedElement') or \
            ch.is_subclass(object_name.namespace,
                  sub=object_name.classname,
                  super=self.job_manager.job_classname):
            return self.simple_refs(env, object_name, model,
                          result_class_name, role, result_role, keys_only)

class LMI_AffectedJobElement(CIMProvider2):
    """ Instrumentation of LMI_AffectedJobElement class and its subclasses."""

    @cmpi_logging.trace_method
    def __init__(self, classname, job_manager):
        self.classname = classname
        self.job_manager = job_manager

    @cmpi_logging.trace_method
    def get_instance(self, env, model):
        """Return an instance."""
        instance_id = model['AffectingElement']['InstanceID']
        job = self.job_manager.get_job_for_instance_id(instance_id)
        if not job:
            raise pywbem.CIMError(pywbem.CIM_ERR_NOT_FOUND,
                    "AffectingElement not found.")

        if job.affected_elements is None:
            raise pywbem.CIMError(pywbem.CIM_ERR_NOT_FOUND,
                    "The AffectingElement has no AffectedElement.")
        if model['AffectedElement'] not in job.affected_elements:
            raise pywbem.CIMError(pywbem.CIM_ERR_NOT_FOUND,
                    "AffectedElement is not associated to AffectingElement.")
        model['ElementEffects'] = [self.Values.ElementEffects.Unknown, ]
        return model

    @cmpi_logging.trace_method
    def enum_instances(self, env, model, keys_only):
        """Enumerate instances."""
        model.path.update({'AffectingElement': None, 'AffectedElement': None})
        for job in self.job_manager.jobs.values():
            if job.affected_elements is None:
                continue
            for element in job.affected_elements:
                model['AffectingElement'] = job.get_name()
                model['AffectedElement'] = element
                if keys_only:
                    yield model
                else:
                    yield self.get_instance(env, model)

    @cmpi_logging.trace_method
    def references(self, env, object_name, model, result_class_name, role,
                   result_role, keys_only):
        """Instrument Associations."""
        ch = env.get_cimom_handle()
        if ch.is_subclass(object_name.namespace,
                  sub=object_name.classname,
                  super='CIM_ManagedElement') or \
            ch.is_subclass(object_name.namespace,
                  sub=object_name.classname,
                  super=self.job_manager.job_classname):
            return self.simple_refs(env, object_name, model,
                          result_class_name, role, result_role, keys_only)

    class Values(object):
        class ElementEffects(object):
            Unknown = pywbem.Uint16(0)
            Other = pywbem.Uint16(1)
            Exclusive_Use = pywbem.Uint16(2)
            Performance_Impact = pywbem.Uint16(3)
            Element_Integrity = pywbem.Uint16(4)
            Create = pywbem.Uint16(5)


class LMI_MethodResult(CIMProvider2):
    """Instrumentation of LMI_MethodResult class and its subclasses."""

    @cmpi_logging.trace_method
    def __init__(self, classname, job_manager):
        self.classname = classname
        self.job_manager = job_manager

    @cmpi_logging.trace_method
    # pylint: disable-msg=W0221
    def get_instance(self, env, model, job=None):
        """Return an instance."""
        if not job:
            instance_id = model['InstanceID']
            job = self.job_manager.get_job_for_instance_id(
                    instance_id, self.classname)
        if not job:
            raise pywbem.CIMError(pywbem.CIM_ERR_NOT_FOUND,
                    "Job not found.")

        model['Description'] = job.job_name
        if job.job_state in Job.FINAL_STATES:
            model['PostCallIndication'] = pywbem.CIMProperty(
                    name='PostCallIndication',
                    value=job.get_post_call())
        else:
            model['PostCallIndication'] = pywbem.CIMProperty(
                    name='PostCallIndication',
                    type='instance',
                    value=None)
        model['PreCallIndication'] = pywbem.CIMProperty(
                    name='PreCallIndication',
                    value=job.get_pre_call())
        return model

    @cmpi_logging.trace_method
    def enum_instances(self, env, model, keys_only):
        """Enumerate instances."""
        model.path.update({'InstanceID': None})
        for job in self.job_manager.jobs.values():
            model['InstanceID'] = job.get_instance_id(
                    classname=self.classname)
            if keys_only:
                yield model
            else:
                yield self.get_instance(env, model, job)

class LMI_AssociatedJobMethodResult(CIMProvider2):
    """
        Instrumentation of LMI_AssociatedJobMethodResult class and its
        subclasses.
    """

    @cmpi_logging.trace_method
    def __init__(self, classname, job_manager):
        self.classname = classname
        self.job_manager = job_manager

    @cmpi_logging.trace_method
    def get_instance(self, env, model):
        """Return an instance."""
        instance_id = model['Job']['InstanceID']
        job = self.job_manager.get_job_for_instance_id(instance_id)
        if not job:
            raise pywbem.CIMError(pywbem.CIM_ERR_NOT_FOUND,
                    "Job not found.")

        expected_result_id = job.get_instance_id(
                classname=self.job_manager.method_result_classname)
        if model['JobParameters']['InstanceID'] != expected_result_id:
            raise pywbem.CIMError(pywbem.CIM_ERR_NOT_FOUND,
                    "Job is not associated to JobParameters.")
        return model

    @cmpi_logging.trace_method
    def enum_instances(self, env, model, keys_only):
        """Enumerate instances."""
        model.path.update({'JobParameters': None, 'Job': None})
        for job in self.job_manager.jobs.values():
            if job.owning_element:
                model['Job'] = job.get_name()
                model['JobParameters'] = pywbem.CIMInstanceName(
                    classname=self.job_manager.method_result_classname,
                    namespace=self.job_manager.namespace,
                    keybindings={
                        'InstanceID': job.get_instance_id(
                            classname=self.job_manager.method_result_classname)
                })
                yield model

    @cmpi_logging.trace_method
    def references(self, env, object_name, model, result_class_name, role,
                   result_role, keys_only):
        """Instrument Associations."""
        ch = env.get_cimom_handle()
        if ch.is_subclass(object_name.namespace,
                  sub=object_name.classname,
                  super=self.job_manager.method_result_classname) or \
            ch.is_subclass(object_name.namespace,
                  sub=object_name.classname,
                  super=self.job_manager.job_classname):
            return self.simple_refs(env, object_name, model,
                          result_class_name, role, result_role, keys_only)




#!/usr/bin/python
#
# Copyright (C) 2012-2013 Red Hat, Inc.  All rights reserved.
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
#
# Authors: Jan Safranek <jsafrane@redhat.com>
#
# This is a tool to dump all object instances from a remote CIM server
# and draw them using dot.
#
# The dump starts at given class - all instances of this class will be drawn.
# The dump then continues with instances referenced by already drawn instances.
# The dumps stops when it reaches given recursion level.
#
# Generated dot output will have class names in node labels (class names are
# way shorter than object paths). Object paths are shown in 'tooltip' - view
# the generated .svg file in Firefox and point your mouse to a node.
# Hyperlinks lead to specified YAWN instance.
#
# Example:
# ./cim2dot.py -u http://ford.usersys.redhat.com:5988 -n root/ontap \
#              -l 20 \
#              -i "Statistic,Stats,LANEnd,ProtocolEnd,Ethernet,DiskDrive,ONTAP_StorageSystem" \
#              -t "http://localhost/yawn/GetInstance" \
#              -c CIM_StorageExtent \
#  | dot -Tsvg >image.svg
#

import sys
import pywbem
import zlib
import cPickle
import base64
import cgi
import optparse

class CimExporter(object):
    def __init__(self, loader, ignore=None, urltemplate='http://localhost/yawn/GetInstance', titles=None):
        self.loader = loader
        self.ignore = ignore
        self.instanceLabels = {} #path -> label
        self.urltemplate = urltemplate
        self.titles = titles

    def isIgnored(self, instance):
        if self.ignore is None:
            return False
        for i in self.ignore.split(','):
            if instance.classname.find(i) >= 0:
                print >>sys.stderr, "Ignoring", i, ':', instance.classname
                return True
        return False

    def getTitle(self, instance):
        title = instance.classname
        if self.titles is None:
            return title
        for i in self.titles.split(','):
            if instance.has_key(i):
                title +=  "\\n" + instance[i]
        return title
        
    def drawInstance(self, instance):
        if self.isIgnored(instance):
            return
        path = self.loader.getInstancePath(instance)
        name = 'obj' + str(len(self.instanceLabels))
        self.instanceLabels[path] = name
        
        _encodeObject = lambda x: (base64.b64encode(zlib.compress(cPickle.dumps(x, cPickle.HIGHEST_PROTOCOL))))
        params = {'url': self.loader.cliconn.url, 'ns': self.loader.namespace, 'instPath':_encodeObject(instance)}
        url = self.urltemplate + "?" + cgi.urllib.urlencode(params)
        title = self.getTitle(instance)
        print '%s [tooltip="%s",label="%s",URL="%s"];' % (name, path, title, url)
        
    def drawReference(self, reference):
        # find the first and the second CIMInstanceName among keybindings
        vals = reference.keybindings.values()
        i = 0;
        while not isinstance(vals[i], pywbem.CIMInstanceName):
            i = i+1
        src = vals[i]
        i = i+1
        while not isinstance(vals[i], pywbem.CIMInstanceName):
            i = i+1
        dst = vals[i]
        
        if self.isIgnored(src) or self.isIgnored(dst) or self.isIgnored(reference):
            return
        label = reference.classname
        srcName = self.instanceLabels[self.loader.getInstancePath(src)]
        dstName = self.instanceLabels[self.loader.getInstancePath(dst)]
        print '%s -- %s [label="%s"];' % (srcName, dstName, label)
        
    def export(self):
        instances = self.loader.instances.values()
        instances.sort()
        references = self.loader.references.values()
        references.sort()
        for i in instances:
            self.drawInstance(i)
        for r in references:
            self.drawReference(r)

class CimLoader(object):
    def __init__(self, address, classname, levels, user=None, password=None, namespace='root/cimv2', ignore=None):
        self.cliconn = pywbem.WBEMConnection(address, (user, password))
        self.cliconn.default_namespace = namespace
        self.levels = levels
        self.namespace = namespace
        self.classname = classname
        self.instances = {} #path -> CIMInstanceName
        self.queue = {} #path -> level
        self.references = {} #'classname:path-path' -> reference
        self.ignore = ignore
        
    def isIgnored(self, instance):
        if self.ignore is None:
            return False
        for i in self.ignore.split(','):
            if instance.classname.find(i) >= 0:
                print >>sys.stderr, "Skipping", i, ':', instance.classname
                return True
        return False

    def getInstancePath(self, instance):
        instance.host='f16'
        path = str(instance)
        return path.replace('"', "'")
    
    def getReferencePath(self, reference, src, dst):
        label = reference.classname
        srcName = self.getInstancePath(src)
        dstName = self.getInstancePath(dst)
        return label+':'+srcName+'-'+dstName
    
    def addInstances(self, instances, level):
        for i in instances:
            path = self.getInstancePath(i)
            if not self.instances.has_key(path):
                self.instances[path] = i
                self.queue[path] = level

    def addReferences(self, instance, level):
        if self.levels <= level:
            return
        try:
            refs = self.cliconn.ReferenceNames(instance)
        except pywbem.cim_operations.CIMError:
            print >>sys.stderr, 'Error getting references of %s: %s' % (self.getInstancePath(instance),  sys.exc_info()[0])
        else:
            for ref in refs:
                for i in ref.values():
                    if isinstance(i, pywbem.CIMInstanceName) and i.classname != instance.classname:
                        if self.isIgnored(i):
                            continue
                        self.addInstances([i,], level+1)
                        if not (self.references.has_key(self.getReferencePath(ref, instance, i))
                                or self.references.has_key(self.getReferencePath(ref, i, instance))):
                            self.references[self.getReferencePath(ref, instance, i)] = ref

    def load(self):
        # add initial instances
        instances = self.cliconn.EnumerateInstanceNames(self.classname, namespace=self.namespace)
        self.addInstances(instances, 0)
        
        while True:
            try:
                (path, level) = self.queue.popitem()
            except KeyError:
                break
            instance = self.instances[path]
            self.addReferences(instance, level)
            
    def dumpTo(self, filename):
        f =  open(filename, 'w')
        cPickle.dump((self.instances, self.references), f)
        f.close()
        
    def loadFrom(self, filename):
        f =  open(filename, 'r')
        (self.instances, self.references) = cPickle.load(f)
        f.close()

# Parse command line arguments
parser = optparse.OptionParser()
parser.add_option('-u', '--url', action='store', dest='addr', default='http://ford.usersys.redhat.com:5988', help='URL of CIM server, default: http://ford.usersys.redhat.com:5988')
parser.add_option('-c', '--class', action='store', dest='classname', help='Name of class to start with. All instances of this class + (recursively) all referenced classes will be drawn. This option is mandatory.')
parser.add_option('-l', '--levels', action='store', type='int', dest='levels', default='5', help='Number of references to track from the initial class. This option limits level of recursion. Default: 5.')
parser.add_option('-i', '--ignore', action='store', dest='ignore', default=None, help='Comma-separated list of classes to ignore. Classes, which contain these strings, will be ignored and won\'t be drawn (but they and their references will be loaded from the CIM server).')
parser.add_option('-I', '--really-ignore', action='store', dest='reallyignore', default=None, help='Comma-separated list of classes to skip. Classes, which contain these strings, will be ignored and won\'t even retrieved from the CIM server.')
parser.add_option('-k', '--draw-keys', action='store', dest='keys', default=None, help='Comma-separated list of object keys, which will be drawn in instances, if the instance has given key.')
parser.add_option('-n', '--namespace', action='store', dest='namespace', default='root/cimv2', help='CIM namespace. Default: root/cimv2')
parser.add_option('-t', '--template', action='store', dest='template', default='http://localhost/yawn/GetInstance', help='Hyperlink template to YAWN. It must have form of "http://localhost/yawn/GetInstance".')
parser.add_option('-U', '--user', action='store', dest='user', default=None, help='CIM user name')
parser.add_option('-P', '--password', action='store', dest='password', default=None, help='CIM password')
parser.add_option('-W', '--write', action='store', dest='outfile', default=None, help='Write discovered CIM objects and references to given file. All objects are written, even those ignored by --ignore option.')
parser.add_option('-R', '--read', action='store', dest='infile', default=None, help='Read objects from given file instead of remote CIM server. Useful when you just want to update --ignore parameter.')
(options, args) = parser.parse_args()

if not options.classname:
    print "Missing classname."
    parser.print_help()
    sys.exit(1)

l = CimLoader(options.addr, options.classname, options.levels, options.user, options.password, options.namespace, options.reallyignore)
if options.infile:
    l.loadFrom(options.infile)
else:
    l.load()
    
if options.outfile:
    l.dumpTo(options.outfile)


e = CimExporter(l, options.ignore, options.template, options.keys)
print """graph test{
node[shape=box]"""
e.export()
print '}'



#!/usr/bin/python
#
# Copyright (C) 2012-2013 Red Hat, Inc.  All rights reserved.
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
#
# Authors: Roman Rakus <rrakus@redhat.com>
#

import sys
import pywbem

def usage():
  print """Usage: %s <address> <username> <password> <command> [parameter...]
    Connect to CIM server at address and issue a command on the machine.

Available commands and their parameters:
  list_user - list users, parameter is user name or empty for all
  list_group - list groups, parameter is group name or empty for all
  group_members - list members of group, parameter is group name
  delete_user - delete account, needed parameter is account name
  delete_group - delete group, needed parameter is group name
  delete_identity - delete user or group, parameter InstanceID of identity
  create_account - creates a new account, parameter required, user login name
  create_group - creates a new group, parameter required, group name

Example: %s https://127.0.0.1:5989 root redhat list_user""" % (sys.argv[0], sys.argv[0])

if len(sys.argv) < 5:
    usage()
    sys.exit(1)

url = sys.argv[1]

username = None
password = None

if sys.argv[2]:
    username = sys.argv[2]
if sys.argv[3]:
    password = sys.argv[3]

command = sys.argv[4]
parameters = sys.argv[5:]

cliconn = pywbem.WBEMConnection(url, (username, password))

if command == "list_user":
# Listintg users is simple, just query all instances from LMI_Account
# or select only by given Name
  slct = "select * from LMI_Account"
  if parameters:
    for i, parameter in enumerate(parameters):
      if i==0:
        slct += ' where Name = "%s"' % parameter
      else:
        slct += ' or Name = "%s"' % parameter

  instances = cliconn.ExecQuery('WQL', slct)
  for instance in instances:
    print instance.tomof()

elif command == "list_group":
# Listintg gruops is simple, just query all instances from LMI_Group
# or select only by given Name
  slct = "select * from LMI_Group"
  if parameters:
    for i, parameter in enumerate(parameters):
      if i==0:
        slct += ' where Name = "%s"' % parameter
      else:
        slct += ' or Name = "%s"' % parameter


  instances = cliconn.ExecQuery('WQL', slct)
  for instance in instances:
    print instance.tomof()

elif command == "group_members":
# Group members is a bit tricky. You need to select group (LMI_Group)
# by given Name, then you need to select identities (LMI_Identity), which
# are connected through LMI_MemberOfGroup to LMI_Group.
# And finally select all accounts (LMI_Account) which are connected through
# LMI_AssignedAccountIdentity with selected identities (this should be
# 1 to 1)
  if not parameters:
    usage()
    sys.exit(1)

  else:
    groups = cliconn.ExecQuery('WQL', 'select * from LMI_Group where Name = "%s"' % parameters[0])
  for group in groups:
    identities = cliconn.Associators(group.path, AssocClass='LMI_MemberOfGroup')
    for identity in identities:
        accounts = cliconn.Associators(identity.path, AssocClass='LMI_AssignedAccountIdentity')
        for account in accounts:
          print account.tomof()

elif command == "create_account":
# create a new account
# Firstly find system name, which is necessary parameter for method
# then invoke the method
  if not parameters:
    usage()
    sys.exit(1)

  computerSystems = cliconn.ExecQuery('WQL', 'select * from Linux_ComputerSystem')
  if not computerSystems:
    print >>sys.stderr, "No usable Linux_ComputerSystem instance found."
    sys.exit(2)

  if len(computerSystems) > 1:
    print >>sys.stderr, "More than one Linux_ComputerSystem instance found, don't know which to use."
    sys.exit(3)

  lams = cliconn.ExecQuery('WQL', 'select * from LMI_AccountManagementService')[0]

  print cliconn.InvokeMethod("CreateAccount", lams.path,
    Name = parameters[0],
    System = computerSystems[0].path)

elif command == "create_group":
# create a new group
# Firstly find system name, which is necessary parameter for method
# then invoke the method
  if not parameters:
    usage()
    sys.exit(1)

  computerSystems = cliconn.ExecQuery('WQL', 'select * from Linux_ComputerSystem')
  if not computerSystems:
    print >>sys.stderr, "No usable Linux_ComputerSystem instance found."
    sys.exit(2)

  if len(computerSystems) > 1:
    print >>sys.stderr, "More than one Linux_ComputerSystem instance found, don't know which to use."
    sys.exit(3)

  lams = cliconn.ExecQuery('WQL', 'select * from LMI_AccountManagementService')[0]

  print cliconn.InvokeMethod("CreateGroup", lams.path,
    Name = parameters[0],
    System = computerSystems[0].path)

elif command == "delete_user":
# Find user by given name and call DeleteInstance on the instance path
  if not parameters:
    usage()
    sys.exit(1)

  slct = 'select * from LMI_Account where Name = "%s"' % parameters[0]

  instances = cliconn.ExecQuery('WQL', slct)
  if instances:
    print cliconn.DeleteInstance(instances[0].path)
  else:
    print >> sys.stderr, "User does not exist: %s" %parameters[0]

elif command == "delete_user":
# Find user by given name and call DeleteInstance on the instance path
  if not parameters:
    usage()
    sys.exit(1)

  slct = 'select * from LMI_Account where Name = "%s"' % parameters[0]

  instances = cliconn.ExecQuery('WQL', slct)
  if instances:
    print cliconn.DeleteInstance(instances[0].path)
  else:
    print >> sys.stderr, "User does not exist: %s" %parameters[0]

elif command == "delete_group":
# Find group by given name and call DeleteInstance on the instance path
  if not parameters:
    usage()
    sys.exit(1)

  slct = 'select * from LMI_Group where Name = "%s"' % parameters[0]

  instances = cliconn.ExecQuery('WQL', slct)
  if instances:
    print cliconn.DeleteInstance(instances[0].path)
  else:
    print >> sys.stderr, "Group does not exist: %s" %parameters[0]

elif command == "delete_identity":
# Have to pass correct InstanceID
# It is in format:
# LMI:UID:user_id
# or
# LMI:GID:group_id
  if not parameters:
    usage()
    sys.exit(1)

  slct = 'select * from LMI_Identity where InstanceID = "%s"' % parameters[0]

  instances = cliconn.ExecQuery('WQL', slct)
  if instances:
    print cliconn.DeleteInstance(instances[0].path)
  else:
    print >> sys.stderr, "Identity does not exist: %s" %parameters[0]

else:
# unknown command
  print "Unknown command", command
  usage()
  sys.exit(1)



# Software Management Providers
#
# Copyright (C) 2012-2013 Red Hat, Inc.  All rights reserved.
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
#
# Authors: Michal Minar <miminar@redhat.com>
#

"""Python Provider for LMI_SoftwareIdentityFileCheck

Instruments the CIM class LMI_SoftwareIdentityFileCheck

"""

import pywbem
from pywbem.cim_provider2 import CIMProvider2

from lmi.providers import cmpi_logging
from lmi.providers import ComputerSystem
from lmi.software.core import IdentityFileCheck

class LMI_SoftwareIdentityFileCheck(CIMProvider2):
    """Instrument the CIM class LMI_SoftwareIdentityFileCheck

    Identifies a file contained by RPM package. It's located in directory
    identified in FileName. The Invoke methods check for file existence
    and whether its attributes match those given by RPM package.

    """

    def __init__(self, _env):
        self.values = IdentityFileCheck.Values

    @cmpi_logging.trace_method
    def get_instance(self, env, model):
        """Return an instance.

        Keyword arguments:
        env -- Provider Environment (pycimmb.ProviderEnvironment)
        model -- A template of the pywbem.CIMInstance to be returned.  The
            key properties are set on this instance to correspond to the
            instanceName that was requested.  The properties of the model
            are already filtered according to the PropertyList from the
            request.  Only properties present in the model need to be
            given values.  If you prefer, you can set all of the
            values, and the instance will be filtered for you.

        Possible Errors:
        CIM_ERR_ACCESS_DENIED
        CIM_ERR_INVALID_PARAMETER (including missing, duplicate, unrecognized
            or otherwise incorrect parameters)
        CIM_ERR_NOT_FOUND (the CIM Class does exist, but the requested CIM
            Instance does not exist in the specified namespace)
        CIM_ERR_FAILED (some other unspecified error occurred)

        """
        file_check = IdentityFileCheck.object_path2file_check(model.path)
        return IdentityFileCheck.file_check2model(file_check, keys_only=False,
                model=model)

    @cmpi_logging.trace_method
    def enum_instances(self, env, model, keys_only):
        """Enumerate instances.

        The WBEM operations EnumerateInstances and EnumerateInstanceNames
        are both mapped to this method.
        This method is a python generator

        Keyword arguments:
        env -- Provider Environment (pycimmb.ProviderEnvironment)
        model -- A template of the pywbem.CIMInstances to be generated.
            The properties of the model are already filtered according to
            the PropertyList from the request.  Only properties present in
            the model need to be given values.  If you prefer, you can
            always set all of the values, and the instance will be filtered
            for you.
        keys_only -- A boolean.  True if only the key properties should be
            set on the generated instances.

        Possible Errors:
        CIM_ERR_FAILED (some other unspecified error occurred)

        """
        # this won't be supported because of enormous amount of data
        # all installed files of all installed rpms would have to be enumerated
        raise pywbem.CIMError(pywbem.CIM_ERR_NOT_SUPPORTED)

    @cmpi_logging.trace_method
    def set_instance(self, env, instance, modify_existing):
        """Return a newly created or modified instance.

        Keyword arguments:
        env -- Provider Environment (pycimmb.ProviderEnvironment)
        instance -- The new pywbem.CIMInstance.  If modifying an existing
            instance, the properties on this instance have been filtered by
            the PropertyList from the request.
        modify_existing -- True if ModifyInstance, False if CreateInstance

        Return the new instance.  The keys must be set on the new instance.

        Possible Errors:
        CIM_ERR_ACCESS_DENIED
        CIM_ERR_NOT_SUPPORTED
        CIM_ERR_INVALID_PARAMETER (including missing, duplicate, unrecognized
            or otherwise incorrect parameters)
        CIM_ERR_ALREADY_EXISTS (the CIM Instance already exists -- only
            valid if modify_existing is False, indicating that the operation
            was CreateInstance)
        CIM_ERR_NOT_FOUND (the CIM Instance does not exist -- only valid
            if modify_existing is True, indicating that the operation
            was ModifyInstance)
        CIM_ERR_FAILED (some other unspecified error occurred)

        """
        raise pywbem.CIMError(pywbem.CIM_ERR_NOT_SUPPORTED)

    @cmpi_logging.trace_method
    def delete_instance(self, env, instance_name):
        """Delete an instance.

        Keyword arguments:
        env -- Provider Environment (pycimmb.ProviderEnvironment)
        instance_name -- A pywbem.CIMInstanceName specifying the instance
            to delete.

        Possible Errors:
        CIM_ERR_ACCESS_DENIED
        CIM_ERR_NOT_SUPPORTED
        CIM_ERR_INVALID_NAMESPACE
        CIM_ERR_INVALID_PARAMETER (including missing, duplicate, unrecognized
            or otherwise incorrect parameters)
        CIM_ERR_INVALID_CLASS (the CIM Class does not exist in the specified
            namespace)
        CIM_ERR_NOT_FOUND (the CIM Class does exist, but the requested CIM
            Instance does not exist in the specified namespace)
        CIM_ERR_FAILED (some other unspecified error occurred)

        """
        raise pywbem.CIMError(pywbem.CIM_ERR_NOT_SUPPORTED)

    @cmpi_logging.trace_method
    def cim_method_invoke(self, env, object_name):
        """Implements LMI_SoftwareIdentityFileCheck.Invoke()

        The Invoke method evaluates this Check. The details of the
        evaluation are described by the specific subclasses of CIM_Check.
        When the SoftwareElement being checked is already installed, the
        CIM_InstalledSoftwareElement association identifies the
        CIM_ComputerSystem in whose context the Invoke is executed. If
        this association is not in place, then the InvokeOnSystem method
        should be used - since it identifies the TargetSystem as an input
        parameter of the method. \nThe results of the Invoke method are
        based on the return value. A zero is returned if the condition is
        satisfied. A one is returned if the method is not supported. Any
        other value indicates the condition is not satisfied.

        Keyword arguments:
        env -- Provider Environment (pycimmb.ProviderEnvironment)
        object_name -- A pywbem.CIMInstanceName or pywbem.CIMCLassName
            specifying the object on which the method Invoke()
            should be invoked.

        Returns a two-tuple containing the return value (type pywbem.Uint32)
        and a list of CIMParameter objects representing the output parameters

        Output parameters: none

        Possible Errors:
        CIM_ERR_ACCESS_DENIED
        CIM_ERR_INVALID_PARAMETER (including missing, duplicate,
            unrecognized or otherwise incorrect parameters)
        CIM_ERR_NOT_FOUND (the target CIM Class or instance does not
            exist in the specified namespace)
        CIM_ERR_METHOD_NOT_AVAILABLE (the CIM Server is unable to honor
            the invocation request)
        CIM_ERR_FAILED (some other unspecified error occurred)

        """
        file_check = IdentityFileCheck.object_path2file_check(object_name)
        out_params = []
        return (      self.values.Invoke.Satisfied
                 if   IdentityFileCheck.file_check_passed(file_check)
                 else self.values.Invoke.Not_Satisfied,
               out_params)

    @cmpi_logging.trace_method
    def cim_method_invokeonsystem(self, env, object_name,
            param_targetsystem=None):
        """Implements LMI_SoftwareIdentityFileCheck.InvokeOnSystem()

        The InvokeOnSystem method evaluates this Check. The details of the
        evaluation are described by the specific subclasses of CIM_Check.
        The method\'s TargetSystem input parameter specifies the
        ComputerSystem in whose context the method is invoked. \nThe
        results of the InvokeOnSystem method are based on the return
        value. A zero is returned if the condition is satisfied. A one is
        returned if the method is not supported. Any other value indicates
        the condition is not satisfied.

        Keyword arguments:
        env -- Provider Environment (pycimmb.ProviderEnvironment)
        object_name -- A pywbem.CIMInstanceName or pywbem.CIMCLassName
            specifying the object on which the method InvokeOnSystem()
            should be invoked.
        param_targetsystem --  The input parameter TargetSystem (
                type REF (pywbem.CIMInstanceName(
                classname='CIM_ComputerSystem', ...))
            Reference to ComputerSystem in whose context the method is to
            be invoked.


        Returns a two-tuple containing the return value (type pywbem.Uint32)
        and a list of CIMParameter objects representing the output parameters

        Output parameters: none

        Possible Errors:
        CIM_ERR_ACCESS_DENIED
        CIM_ERR_INVALID_PARAMETER (including missing, duplicate,
            unrecognized or otherwise incorrect parameters)
        CIM_ERR_NOT_FOUND (the target CIM Class or instance does not
            exist in the specified namespace)
        CIM_ERR_METHOD_NOT_AVAILABLE (the CIM Server is unable to honor
            the invocation request)
        CIM_ERR_FAILED (some other unspecified error occurred)
        """
        if param_targetsystem is not None:
            ComputerSystem.check_path(env, param_targetsystem, "TargetSystem")
        return self.cim_method_invoke(env, object_name)




# Copyright (C) 2013 Red Hat, Inc.  All rights reserved.
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
#
# Authors: Jan Synacek <jsynacek@redhat.com>

import os
import subprocess
import pyudev

from lmi.test import cimbase

class LogicalFileTestBase(cimbase.CIMTestCase):
    """
    Base class for all LogicalFile tests.
    """

    @classmethod
    def setUpClass(cls):
        cimbase.CIMTestCase.setUpClass.im_func(cls)
        cls.testdir = os.environ.get("LMI_LOGICALFILE_TESTDIR", "/var/tmp/logicalfile-tests")
        cls.selinux_enabled = True
        try:
            rc = subprocess.call(['selinuxenabled'])
            if rc != 0: cls.selinux_enabled = False
        except:
            cls.selinux_enabled = False
        ctx = pyudev.Context()
        sb = os.stat(os.path.realpath(cls.testdir + "/.."))
        device = pyudev.Device.from_device_number(ctx, "block", sb.st_dev)
        dev_name = device.get("ID_FS_UUID_ENC")
        if not dev_name:
            cls.fsname = "DEVICE=" + device.get("DEVNAME")
        else:
            cls.fsname = "UUID=" + dev_name

    def setUp(self):
        pass

    def tearDown(self):
        pass



# Software Management Providers
#
# Copyright (C) 2012-2013 Red Hat, Inc.  All rights reserved.
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
#
# Authors: Michal Minar <miminar@redhat.com>
#

"""Python Provider for LMI_SoftwareIdentityResource

Instruments the CIM class LMI_SoftwareIdentityResource

"""

import pywbem
from pywbem.cim_provider2 import CIMProvider2

from lmi.providers import cmpi_logging
from lmi.software.core import IdentityResource
from lmi.software.yumdb import YumDB, errors

LOG = cmpi_logging.get_logger(__name__)

class LMI_SoftwareIdentityResource(CIMProvider2):
    """Instrument the CIM class LMI_SoftwareIdentityResource

    SoftwareIdentityResource describes the URL of a file or other resource
    that contains all or part of of a SoftwareIdentity for use by the
    SoftwareInstallationService. For example, a CIM_SoftwareIdentity might
    consist of a meta data file, a binary executable file, and a
    installability checker file for some software on a system. This class
    allows a management client to selectively access the constituents of
    the install package to perform a check function, or retrieve some meta
    data information for the install package represented by the
    SoftwareIdentity class without downloading the entire package.
    SoftwareIdentityResources will be related to the SoftwareIdentity
    using the SAPAvailableForElement association.

    """

    def __init__(self, _env):
        self.values = IdentityResource.Values

    @cmpi_logging.trace_method
    def get_instance(self, env, model):
        """Return an instance.

        Keyword arguments:
        env -- Provider Environment (pycimmb.ProviderEnvironment)
        model -- A template of the pywbem.CIMInstance to be returned.  The
            key properties are set on this instance to correspond to the
            instanceName that was requested.  The properties of the model
            are already filtered according to the PropertyList from the
            request.  Only properties present in the model need to be
            given values.  If you prefer, you can set all of the
            values, and the instance will be filtered for you.

        Possible Errors:
        CIM_ERR_ACCESS_DENIED
        CIM_ERR_INVALID_PARAMETER (including missing, duplicate, unrecognized
            or otherwise incorrect parameters)
        CIM_ERR_NOT_FOUND (the CIM Class does exist, but the requested CIM
            Instance does not exist in the specified namespace)
        CIM_ERR_FAILED (some other unspecified error occurred)
        """
        repo = IdentityResource.object_path2repo(env, model.path, kind='all')
        return IdentityResource.repo2model(env, repo, keys_only=False, model=model)

    @cmpi_logging.trace_method
    def enum_instances(self, env, model, keys_only):
        """Enumerate instances.

        The WBEM operations EnumerateInstances and EnumerateInstanceNames
        are both mapped to this method.
        This method is a python generator

        Keyword arguments:
        env -- Provider Environment (pycimmb.ProviderEnvironment)
        model -- A template of the pywbem.CIMInstances to be generated.
            The properties of the model are already filtered according to
            the PropertyList from the request.  Only properties present in
            the model need to be given values.  If you prefer, you can
            always set all of the values, and the instance will be filtered
            for you.
        keys_only -- A boolean.  True if only the key properties should be
            set on the generated instances.

        Possible Errors:
        CIM_ERR_FAILED (some other unspecified error occurred)

        """
        # Prime model.path with knowledge of the keys, so key values on
        # the CIMInstanceName (model.path) will automatically be set when
        # we set property values on the model.
        model.path.update({'CreationClassName': None, 'SystemName': None,
            'Name': None, 'SystemCreationClassName': None})

        repolist = YumDB.get_instance().get_repository_list('all')
        for repo in repolist:
            yield IdentityResource.repo2model(
                    env, repo, keys_only=keys_only, model=model)

    @cmpi_logging.trace_method
    def set_instance(self, env, instance, modify_existing):
        """Return a newly created or modified instance.

        Keyword arguments:
        env -- Provider Environment (pycimmb.ProviderEnvironment)
        instance -- The new pywbem.CIMInstance.  If modifying an existing
            instance, the properties on this instance have been filtered by
            the PropertyList from the request.
        modify_existing -- True if ModifyInstance, False if CreateInstance

        Return the new instance.  The keys must be set on the new instance.

        Possible Errors:
        CIM_ERR_ACCESS_DENIED
        CIM_ERR_NOT_SUPPORTED
        CIM_ERR_INVALID_PARAMETER (including missing, duplicate, unrecognized
            or otherwise incorrect parameters)
        CIM_ERR_ALREADY_EXISTS (the CIM Instance already exists -- only
            valid if modify_existing is False, indicating that the operation
            was CreateInstance)
        CIM_ERR_NOT_FOUND (the CIM Instance does not exist -- only valid
            if modify_existing is True, indicating that the operation
            was ModifyInstance)
        CIM_ERR_FAILED (some other unspecified error occurred)

        """
        # TODO creation and modification should be supported
        raise pywbem.CIMError(pywbem.CIM_ERR_NOT_SUPPORTED)

    @cmpi_logging.trace_method
    def delete_instance(self, env, instance_name):
        """Delete an instance.

        Keyword arguments:
        env -- Provider Environment (pycimmb.ProviderEnvironment)
        instance_name -- A pywbem.CIMInstanceName specifying the instance
            to delete.

        Possible Errors:
        CIM_ERR_ACCESS_DENIED
        CIM_ERR_NOT_SUPPORTED
        CIM_ERR_INVALID_NAMESPACE
        CIM_ERR_INVALID_PARAMETER (including missing, duplicate, unrecognized
            or otherwise incorrect parameters)
        CIM_ERR_INVALID_CLASS (the CIM Class does not exist in the specified
            namespace)
        CIM_ERR_NOT_FOUND (the CIM Class does exist, but the requested CIM
            Instance does not exist in the specified namespace)
        CIM_ERR_FAILED (some other unspecified error occurred)

        """
        # TODO removal should also be supported
        raise pywbem.CIMError(pywbem.CIM_ERR_NOT_SUPPORTED)

    @cmpi_logging.trace_method
    def cim_method_requeststatechange(self, env, object_name,
                                      param_requestedstate=None,
                                      param_timeoutperiod=None):
        """Implements LMI_SoftwareIdentityResource.RequestStateChange()

        Requests that the state of the element be changed to the value
        specified in the RequestedState parameter. When the requested
        state change takes place, the EnabledState and RequestedState of
        the element will be the same. Invoking the RequestStateChange
        method multiple times could result in earlier requests being
        overwritten or lost.  A return code of 0 shall indicate the state
        change was successfully initiated.  A return code of 3 shall
        indicate that the state transition cannot complete within the
        interval specified by the TimeoutPeriod parameter.  A return code
        of 4096 (0x1000) shall indicate the state change was successfully
        initiated, a ConcreteJob has been created, and its reference
        returned in the output parameter Job. Any other return code
        indicates an error condition.

        Keyword arguments:
        env -- Provider Environment (pycimmb.ProviderEnvironment)
        object_name -- A pywbem.CIMInstanceName or pywbem.CIMCLassName
            specifying the object on which the method RequestStateChange()
            should be invoked.
        param_requestedstate --  The input parameter RequestedState (
                type pywbem.Uint16 Values.RequestStateChange.RequestedState)
            The state requested for the element. This information will be
            placed into the RequestedState property of the instance if the
            return code of the RequestStateChange method is 0 ('Completed
            with No Error'), or 4096 (0x1000) ('Job Started'). Refer to
            the description of the EnabledState and RequestedState
            properties for the detailed explanations of the RequestedState
            values.

        param_timeoutperiod --  The input parameter TimeoutPeriod (
                type pywbem.CIMDateTime)
            A timeout period that specifies the maximum amount of time that
            the client expects the transition to the new state to take.
            The interval format must be used to specify the TimeoutPeriod.
            A value of 0 or a null parameter indicates that the client has
            no time requirements for the transition.  If this property
            does not contain 0 or null and the implementation does not
            support this parameter, a return code of 'Use Of Timeout
            Parameter Not Supported' shall be returned.


        Returns a two-tuple containing the return value (
            type pywbem.Uint32 Values.RequestStateChange)
        and a list of CIMParameter objects representing the output parameters

        Output parameters:
        Job -- (type REF (pywbem.CIMInstanceName(
                    classname='CIM_ConcreteJob', ...))
            May contain a reference to the ConcreteJob created to track the
            state transition initiated by the method invocation.

        Possible Errors:
        CIM_ERR_ACCESS_DENIED
        CIM_ERR_INVALID_PARAMETER (including missing, duplicate,
            unrecognized or otherwise incorrect parameters)
        CIM_ERR_NOT_FOUND (the target CIM Class or instance does not
            exist in the specified namespace)
        CIM_ERR_METHOD_NOT_AVAILABLE (the CIM Server is unable to honor
            the invocation request)
        CIM_ERR_FAILED (some other unspecified error occurred)
        """
        out_params = []
        if param_timeoutperiod is not None:
            return ( self.values.RequestStateChange. \
                        Use_of_Timeout_Parameter_Not_Supported
                   , out_params)
        if param_requestedstate not in {
                self.values.RequestStateChange.RequestedState.Enabled,
                self.values.RequestStateChange.RequestedState.Disabled }:
            return ( self.values.RequestStateChange.Invalid_State_Transition
                   , out_params)

        try:
            with YumDB.get_instance() as ydb:
                repo = IdentityResource.object_path2repo(env,
                        object_name, 'all')
                enable = param_requestedstate == \
                        self.values.RequestStateChange.RequestedState.Enabled
                LOG().info("%s repository %s" % ("enabling"
                    if enable else "disabling", repo))
                try:
                    prev = ydb.set_repository_enabled(
                            repo, enable)
                except errors.RepositoryChangeError as exc:
                    msg = "failed to modify repository %s: %s" % (repo, str(exc))
                    LOG().error(msg)
                    raise pywbem.CIMError(pywbem.CIM_ERR_FAILED, msg)
                msg = ( "repository %s already %s"
                      if prev == enable else "repository %s %s")
                LOG().info(msg % (repo, "enabled" if enable else "disabled"))
        except errors.RepositoryNotFound as err:
            raise pywbem.CIMError(pywbem.CIM_ERR_NOT_FOUND, err.message)

        rval = self.values.RequestStateChange.Completed_with_No_Error
        return (rval, out_params)




# Software Management Providers
#
# Copyright (C) 2012-2013 Red Hat, Inc.  All rights reserved.
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA

"""Python Provider for LMI_SoftwareInstallationServiceCapabilities

Instruments the CIM class LMI_SoftwareInstallationServiceCapabilities

"""

import pywbem
from pywbem.cim_provider2 import CIMProvider2

from lmi.providers import cmpi_logging
from lmi.software.core import InstallationServiceCapabilities

class LMI_SoftwareInstallationServiceCapabilities(CIMProvider2):
    """Instrument the CIM class LMI_SoftwareInstallationServiceCapabilities

    A subclass of capabilities that defines the capabilities of a
    SoftwareInstallationService. A single instance of
    SoftwareInstallationServiceCapabilities is associated with a
    SoftwareInstallationService using ElementCapabilities.
    """

    def __init__ (self, _env):
        self.values = InstallationServiceCapabilities.Values

    @cmpi_logging.trace_method
    def get_instance(self, env, model):
        """Return an instance.

        Keyword arguments:
        env -- Provider Environment (pycimmb.ProviderEnvironment)
        model -- A template of the pywbem.CIMInstance to be returned.  The
            key properties are set on this instance to correspond to the
            instanceName that was requested.  The properties of the model
            are already filtered according to the PropertyList from the
            request.  Only properties present in the model need to be
            given values.  If you prefer, you can set all of the
            values, and the instance will be filtered for you.

        Possible Errors:
        CIM_ERR_ACCESS_DENIED
        CIM_ERR_INVALID_PARAMETER (including missing, duplicate, unrecognized
            or otherwise incorrect parameters)
        CIM_ERR_NOT_FOUND (the CIM Class does exist, but the requested CIM
            Instance does not exist in the specified namespace)
        CIM_ERR_FAILED (some other unspecified error occurred)
        """
        InstallationServiceCapabilities.check_path(env,
                model.path, "path")
        return InstallationServiceCapabilities.get_instance(model)

    @cmpi_logging.trace_method
    def enum_instances(self, env, model, keys_only):
        """Enumerate instances.

        The WBEM operations EnumerateInstances and EnumerateInstanceNames
        are both mapped to this method.
        This method is a python generator

        Keyword arguments:
        env -- Provider Environment (pycimmb.ProviderEnvironment)
        model -- A template of the pywbem.CIMInstances to be generated.
            The properties of the model are already filtered according to
            the PropertyList from the request.  Only properties present in
            the model need to be given values.  If you prefer, you can
            always set all of the values, and the instance will be filtered
            for you.
        keys_only -- A boolean.  True if only the key properties should be
            set on the generated instances.

        Possible Errors:
        CIM_ERR_FAILED (some other unspecified error occurred)
        """
        model.path.update({'InstanceID': None})

        if keys_only:
            model['InstanceID'] = \
                    InstallationServiceCapabilities.get_path()['InstanceID']
        else:
            model = InstallationServiceCapabilities.get_instance(model)
        yield model

    @cmpi_logging.trace_method
    def set_instance(self, env, instance, modify_existing):
        """Return a newly created or modified instance.

        Keyword arguments:
        env -- Provider Environment (pycimmb.ProviderEnvironment)
        instance -- The new pywbem.CIMInstance.  If modifying an existing
            instance, the properties on this instance have been filtered by
            the PropertyList from the request.
        modify_existing -- True if ModifyInstance, False if CreateInstance

        Return the new instance.  The keys must be set on the new instance.

        Possible Errors:
        CIM_ERR_ACCESS_DENIED
        CIM_ERR_NOT_SUPPORTED
        CIM_ERR_INVALID_PARAMETER (including missing, duplicate, unrecognized
            or otherwise incorrect parameters)
        CIM_ERR_ALREADY_EXISTS (the CIM Instance already exists -- only
            valid if modify_existing is False, indicating that the operation
            was CreateInstance)
        CIM_ERR_NOT_FOUND (the CIM Instance does not exist -- only valid
            if modify_existing is True, indicating that the operation
            was ModifyInstance)
        CIM_ERR_FAILED (some other unspecified error occurred)
        """
        raise pywbem.CIMError(pywbem.CIM_ERR_NOT_SUPPORTED)

    @cmpi_logging.trace_method
    def delete_instance(self, env, instance_name):
        """Delete an instance.

        Keyword arguments:
        env -- Provider Environment (pycimmb.ProviderEnvironment)
        instance_name -- A pywbem.CIMInstanceName specifying the instance
            to delete.

        Possible Errors:
        CIM_ERR_ACCESS_DENIED
        CIM_ERR_NOT_SUPPORTED
        CIM_ERR_INVALID_NAMESPACE
        CIM_ERR_INVALID_PARAMETER (including missing, duplicate, unrecognized
            or otherwise incorrect parameters)
        CIM_ERR_INVALID_CLASS (the CIM Class does not exist in the specified
            namespace)
        CIM_ERR_NOT_FOUND (the CIM Class does exist, but the requested CIM
            Instance does not exist in the specified namespace)
        CIM_ERR_FAILED (some other unspecified error occurred)
        """
        raise pywbem.CIMError(pywbem.CIM_ERR_NOT_SUPPORTED)

    @cmpi_logging.trace_method
    def cim_method_creategoalsettings(self, env, object_name,
            param_supportedgoalsettings=None,
            param_templategoalsettings=None):
        """Implements LMI_SoftwareInstallationServiceCapabilities. \
                CreateGoalSettings()

        Method to create a set of supported SettingData elements, from two
        sets of SettingData elements, provided by the caller.  CreateGoal
        should be used when the SettingData instances that represents the
        goal will not persist beyond the execution of the client and where
        those instances are not intended to be shared with other,
        non-cooperating clients.  Both TemplateGoalSettings and
        SupportedGoalSettings are represented as strings containing
        EmbeddedInstances of a CIM_SettingData subclass. These embedded
        instances do not exist in the infrastructure supporting this
        method but are maintained by the caller/client.  This method
        should return CIM_Error(s) representing that a single named
        property of a setting (or other) parameter (either reference or
        embedded object) has an invalid value or that an invalid
        combination of named properties of a setting (or other) parameter
        (either reference or embedded object) has been requested.  If the
        input TemplateGoalSettings is NULL or the empty string, this
        method returns a default SettingData element that is supported by
        this Capabilities element.  If the TemplateGoalSettings specifies
        values that cannot be supported, this method shall return an
        appropriate CIM_Error and should return a best match for a
        SupportedGoalSettings.  The client proposes a goal using the
        TemplateGoalSettings parameter and gets back Success if the
        TemplateGoalSettings is exactly supportable. It gets back
        "Alternative Proposed" if the output SupportedGoalSettings
        represents a supported alternative. This alternative should be a
        best match, as defined by the implementation.  If the
        implementation is conformant to a RegisteredProfile, then that
        profile may specify the algorithms used to determine best match. A
        client may compare the returned value of each property against the
        requested value to determine if it is left unchanged, degraded or
        upgraded.   Otherwise, if the TemplateGoalSettings is not
        applicable an "Invalid Parameter" error is returned.   When a
        mutually acceptable SupportedGoalSettings has been achieved, the
        client may use the contained SettingData instances as input to
        methods for creating a new object ormodifying an existing object.
        Also the embedded SettingData instances returned in the
        SupportedGoalSettings may be instantiated via CreateInstance,
        either by a client or as a side-effect of the execution of an
        extrinsic method for which the returned SupportedGoalSettings is
        passed as an embedded instance.

        Keyword arguments:
        env -- Provider Environment (pycimmb.ProviderEnvironment)
        object_name -- A pywbem.CIMInstanceName or pywbem.CIMCLassName
            specifying the object on which the method CreateGoalSettings()
            should be invoked.
        param_supportedgoalsettings --  The input parameter
                SupportedGoalSettings (type pywbem.CIMInstance(
                    classname='CIM_SettingData', ...))
            SupportedGoalSettings are elements of class CIM_SettingData, or
            a derived class.  At most, one instance of each SettingData
            subclass may be supplied.  All SettingData instances provided
            by this property are interpreted as a set, relative to this
            Capabilities instance.   To enable a client to provide
            additional information towards achieving the
            TemplateGoalSettings, an input set of SettingData instances
            may be provided. If not provided, this property shall be set
            to NULL on input.. Note that when provided, what property
            values are changed, and how, is implementation dependent and
            may be the subject of other standards.  If provided, the input
            SettingData instances must be ones that the implementation is
            able to support relative to the ManagedElement associated via
            ElementCapabilities. Typically, the input SettingData
            instances are created by a previous instantiation of
            CreateGoalSettings.  If the input SupportedGoalSettings is not
            supported by the implementation, then an "Invalid Parameter"
            (5) error is returned by this call. In this case, a
            corresponding CIM_ERROR should also be returned.  On output,
            this property is used to return the best supported match to
            the TemplateGoalSettings.  If the output SupportedGoalSettings
            matches the input SupportedGoalSettings, then the
            implementation is unable to improve further towards meeting
            the TemplateGoalSettings.

        param_templategoalsettings --  The input parameter
                TemplateGoalSettings (type pywbem.CIMInstance(
                    classname='CIM_SettingData', ...))
            If provided, TemplateGoalSettings are elements of class
            CIM_SettingData, or a derived class, that is used as the
            template to be matched. .  At most, one instance of each
            SettingData subclass may be supplied.  All SettingData
            instances provided by this property are interpreted as a set,
            relative to this Capabilities instance.  SettingData instances
            that are not relevant to this instance are ignored.  If not
            provided, it shall be set to NULL. In that case, a SettingData
            instance representing the default settings of the associated
            ManagedElement is used.


        Returns a two-tuple containing the return value (type pywbem.Uint16
            self.Values.CreateGoalSettings)
        and a list of CIMParameter objects representing the output parameters

        Output parameters:
        SupportedGoalSettings -- (type
                pywbem.CIMInstance(classname='CIM_SettingData', ...))
            SupportedGoalSettings are elements of class CIM_SettingData, or
            a derived class.  At most, one instance of each SettingData
            subclass may be supplied.  All SettingData instances provided
            by this property are interpreted as a set, relative to this
            Capabilities instance.   To enable a client to provide
            additional information towards achieving the
            TemplateGoalSettings, an input set of SettingData instances
            may be provided. If not provided, this property shall be set
            to NULL on input.. Note that when provided, what property
            values are changed, and how, is implementation dependent and
            may be the subject of other standards.  If provided, the input
            SettingData instances must be ones that the implementation is
            able to support relative to the ManagedElement associated via
            ElementCapabilities. Typically, the input SettingData
            instances are created by a previous instantiation of
            CreateGoalSettings.  If the input SupportedGoalSettings is not
            supported by the implementation, then an "Invalid Parameter"
            (5) error is returned by this call. In this case, a
            corresponding CIM_ERROR should also be returned.  On output,
            this property is used to return the best supported match to
            the TemplateGoalSettings.  If the output SupportedGoalSettings
            matches the input SupportedGoalSettings, then the
            implementation is unable to improve further towards meeting
            the TemplateGoalSettings.


        Possible Errors:
        CIM_ERR_ACCESS_DENIED
        CIM_ERR_INVALID_PARAMETER (including missing, duplicate,
            unrecognized or otherwise incorrect parameters)
        CIM_ERR_NOT_FOUND (the target CIM Class or instance does not
            exist in the specified namespace)
        CIM_ERR_METHOD_NOT_AVAILABLE (the CIM Server is unable to honor
            the invocation request)
        CIM_ERR_FAILED (some other unspecified error occurred)
        """
        return (self.values.CreateGoalSettings.Not_Supported, [])




# PCP bridge Providers
#
# Copyright (C) 2013 Red Hat, Inc.  All rights reserved.
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
#
# Authors: Frank Ch. Eigler <fche@redhat.com>
#

"""Python Provider for PCP_Metric_*
Instruments the CIM class family PCP_Metric_*
"""

from pywbem.cim_provider2 import CIMProvider2
import pywbem
from pcp import pmapi
import cpmapi as c_api
import datetime

context = None  # persistent pcp context


# Since we serve a whole slew of PCP_Metric_** subclasses, we can't
# use a straight classname->provider-class dictionary.
#
#def get_providers(env): 
#    return {'PCP_Metric_****': PCP_MetricProvider} 
#
# Instead, we implement the one-hop-higher proxy-level function calls,
# namely the IM_* functions at the bottom.


# Undo mangling done by PCP_pmns2mofreg.sh
def MOFname_to_PCPmetric (op):
    assert (op.namespace == 'root/cimv2')
    assert (op.classname[0:11] == 'PCP_Metric_')
    return op.classname[11:].replace('__', '.')


# Search the given PM_ResultSet for a given instance number (if any);
# return formatted CIM of the value (or CIM None)
def PCP_CIMValueString (context, result, desc, inst):
    for i in range (0,result.contents.get_numval(0)):
        value = result.contents.get_vlist(0,i)
        iv = value.inst
        if (inst is not None and inst != iv):
            continue
        atom = context.pmExtractValue (result.contents.get_valfmt(0),
                                       value,
                                       desc.contents.type,
                                       desc.contents.type)
        atomValue = atom.dref(desc.contents.type) # nb: atomValue could be numeric etc.
        return pywbem.CIMProperty(name='ValueString',
                                  value=str(atomValue), # stringify it here
                                  type='string')

    return pywbem.CIMProperty(name='ValueString',
                              value=None,
                              type='string')


def PCP_CIMStatisticTime (result):
    dt = datetime.datetime.fromtimestamp(float(str(result.contents.timestamp)))
    return pywbem.CIMDateTime(dt)


# generic payload generator, used for
# - iterating across instance domains (keys_only=1)
# - fetching metric values
# - fetching metric metadata (for those model/filter fields set)
def get_instance (env, op, model, keys_only):
    metric = MOFname_to_PCPmetric (op)
    global context

    try:
        if (context == None):
            context = pmapi.pmContext() # localhost or equivalent
        context.pmReconnectContext() # in case it was nuked recently
    except pmapi.pmErr, e:
        context = None
        raise pywbem.CIMError(pywbem.CIM_ERR_FAILED, "Unable to connect to local PMCD:" + str(e))
        
    pmids = context.pmLookupName(metric)
    pmid = pmids[0]
    desc = context.pmLookupDesc(pmid)

    if ('InstanceID' in model):
        selected_instanceid = model['InstanceID']
    else:
        selected_instanceid = None
    
    model.path.update({'InstanceID':None})

    if (not keys_only):
        model['PMID'] = pywbem.Uint32(pmid)
        model['ElementName'] = metric
        # must not fail, or else we have no metric value data worth sharing
        try:
            results = context.pmFetch(pmids)
        except pmapi.pmErr, e:
            # fatal
            raise pywbem.CIMError(pywbem.CIM_ERR_FAILED, "PCP pmFetch failed:" + str(e))
        # cannot fail
        model['Units'] = context.pmUnitsStr(desc.contents.units)
        model['Type'] = context.pmTypeStr(desc.contents.type)
        # these may fail, but not fatally
        try:
            model['Caption'] = context.pmLookupText(pmid)
        except pmapi.pmErr:
            pass
        try:
            model['Description'] = context.pmLookupText(pmid, c_api.PM_TEXT_HELP)
        except pmapi.pmErr:
            pass

    try:
        instL, nameL = context.pmGetInDom(desc)
        for iL, nL in zip(instL, nameL):
            new_instanceid = 'PCP:'+metric+':'+nL
            if (selected_instanceid is None or
                new_instanceid == selected_instanceid):
                model['InstanceNumber'] = pywbem.Uint32(iL)
                model['InstanceName'] = nL
                model['InstanceID'] = new_instanceid
                if (not keys_only):
                    model['StatisticTime'] = PCP_CIMStatisticTime (results)
                    model['ValueString'] = PCP_CIMValueString (context, results, desc, iL)
                yield model
    except pmapi.pmErr: # pmGetInDom is expected to fail for non-instance (PM_INDOM_NULL) metrics 
        new_instanceid = 'PCP:'+metric
        if (selected_instanceid is None or
            new_instanceid == selected_instanceid):
            model['InstanceNumber'] = pywbem.CIMProperty(name='InstanceNumber',
                                                         value=None,type='uint32')
            model['InstanceName'] = pywbem.CIMProperty(name='InstanceName',
                                                       value=None,type='string')
            model['InstanceID'] = new_instanceid
            if (not keys_only):
                model['StatisticTime'] = PCP_CIMStatisticTime (results)
                model['ValueString'] = PCP_CIMValueString (context, results, desc, None)
            yield model
        
# hooks for impersonating CIMProvider2 functions

    
def MI_enumInstanceNames (env, op):
    model = pywbem.CIMInstance(classname = op.classname, path=op)
    for x in get_instance (env, op, model, True):
        yield x.path

def MI_enumInstances (env, op, plist):
    model = pywbem.CIMInstance(classname = op.classname, path=op)
    return get_instance (env, op, model, False)

def MI_getInstance (env, op, plist):
    proplist = None
    if plist is not None:
        proplist = [s.lower() for s in propertyList]
        proplist+= [s.lower() for s in op.keybindings.keys()]
    model = pywbem.CIMInstance(classname=op.classname, path=op,
                               property_list=proplist)
    model.update(model.path.keybindings)
    for x in get_instance (env, op, model, False):
        return x # XXX: first one

def MI_createInstance (env, pinst):
    raise pywbem.CIMError(pywbem.CIM_ERR_NOT_SUPPORTED)

def MI_modifyInstance (env, pinst, plist):
    raise pywbem.CIMError(pywbem.CIM_ERR_NOT_SUPPORTED)

def MI_deleteInstance (env, piname):
    raise pywbem.CIMError(pywbem.CIM_ERR_NOT_SUPPORTED)

# See also extra MI_* functions for associations, etc.;
# cmpi-bindings.git swig/python/cmpi_pywbem_bindings.py



# -*- encoding: utf-8 -*-
# Software Management Providers
#
# Copyright (C) 2012-2013 Red Hat, Inc.  All rights reserved.
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA

"""Python Provider for LMI_AssociatedSoftwareJobMethodResult

Instruments the CIM class LMI_AssociatedSoftwareJobMethodResult
"""

import pywbem
from pywbem.cim_provider2 import CIMProvider2

from lmi.providers import cmpi_logging
from lmi.software.core import generate_references
from lmi.software.core import Job
from lmi.software.core import MethodResult
from lmi.software.yumdb import YumDB

@cmpi_logging.trace_function
def generate_job_referents(env, object_name, model, _keys_only):
    """
    Handler for referents enumeration request.
    """
    job = Job.object_path2job(object_name)

    model["Job"] = Job.job2model(job)
    model["JobParameters"] = MethodResult.job2model(env, job)
    yield model

@cmpi_logging.trace_function
def generate_result_referents(env, object_name, model, _keys_only):
    """
    Handler for referents enumeration request.
    """
    job = MethodResult.object_path2job(object_name)

    model["Job"] = Job.job2model(job)
    model["JobParameters"] = MethodResult.job2model(env, job)
    yield model

class LMI_AssociatedSoftwareJobMethodResult(CIMProvider2):
    """Instrument the CIM class LMI_AssociatedSoftwareJobMethodResult

    AssociatedJobMethodResult represents an association between a
    ConcreteJob and the MethodResult expressing the parameters for the Job
    when the job was created by side-effect of the execution of an
    extrinsic method.

    """

    def __init__(self, _env):
        pass

    @cmpi_logging.trace_method
    def get_instance(self, env, model):
        """Return an instance.

        Keyword arguments:
        env -- Provider Environment (pycimmb.ProviderEnvironment)
        model -- A template of the pywbem.CIMInstance to be returned.  The
            key properties are set on this instance to correspond to the
            instanceName that was requested.  The properties of the model
            are already filtered according to the PropertyList from the
            request.  Only properties present in the model need to be
            given values.  If you prefer, you can set all of the
            values, and the instance will be filtered for you.

        Possible Errors:
        CIM_ERR_ACCESS_DENIED
        CIM_ERR_INVALID_PARAMETER (including missing, duplicate, unrecognized
            or otherwise incorrect parameters)
        CIM_ERR_NOT_FOUND (the CIM Class does exist, but the requested CIM
            Instance does not exist in the specified namespace)
        CIM_ERR_FAILED (some other unspecified error occurred)
        """
        job = Job.object_path2job(model['Job'])
        jobid = MethodResult.object_path2jobid(model['JobParameters'])
        if job.jobid != jobid:
            raise pywbem.CIMError(pywbem.CIM_ERR_NOT_FOUND,
                    "Job id of Job(%d) does not match JobParameters(%d)." %
                    (job.jobid, jobid))
        model["Job"] = Job.job2model(job)
        model["JobParameters"] = MethodResult.job2model(env, job)
        return model

    @cmpi_logging.trace_method
    def enum_instances(self, env, model, keys_only):
        """Enumerate instances.

        The WBEM operations EnumerateInstances and EnumerateInstanceNames
        are both mapped to this method.
        This method is a python generator

        Keyword arguments:
        env -- Provider Environment (pycimmb.ProviderEnvironment)
        model -- A template of the pywbem.CIMInstances to be generated.
            The properties of the model are already filtered according to
            the PropertyList from the request.  Only properties present in
            the model need to be given values.  If you prefer, you can
            always set all of the values, and the instance will be filtered
            for you.
        keys_only -- A boolean.  True if only the key properties should be
            set on the generated instances.

        Possible Errors:
        CIM_ERR_FAILED (some other unspecified error occurred)
        """
        model.path.update({'Job': None, 'JobParameters': None})

        for job in YumDB.get_instance().get_job_list():
            model['Job'] = Job.job2model(job)
            model['JobParameters'] = MethodResult.job2model(env, job)
            yield model

    @cmpi_logging.trace_method
    def set_instance(self, env, instance, modify_existing):
        """Return a newly created or modified instance.

        Keyword arguments:
        env -- Provider Environment (pycimmb.ProviderEnvironment)
        instance -- The new pywbem.CIMInstance.  If modifying an existing
            instance, the properties on this instance have been filtered by
            the PropertyList from the request.
        modify_existing -- True if ModifyInstance, False if CreateInstance

        Return the new instance.  The keys must be set on the new instance.

        Possible Errors:
        CIM_ERR_ACCESS_DENIED
        CIM_ERR_NOT_SUPPORTED
        CIM_ERR_INVALID_PARAMETER (including missing, duplicate, unrecognized
            or otherwise incorrect parameters)
        CIM_ERR_ALREADY_EXISTS (the CIM Instance already exists -- only
            valid if modify_existing is False, indicating that the operation
            was CreateInstance)
        CIM_ERR_NOT_FOUND (the CIM Instance does not exist -- only valid
            if modify_existing is True, indicating that the operation
            was ModifyInstance)
        CIM_ERR_FAILED (some other unspecified error occurred)
        """
        raise pywbem.CIMError(pywbem.CIM_ERR_NOT_SUPPORTED)

    @cmpi_logging.trace_method
    def delete_instance(self, env, instance_name):
        """Delete an instance.

        Keyword arguments:
        env -- Provider Environment (pycimmb.ProviderEnvironment)
        instance_name -- A pywbem.CIMInstanceName specifying the instance
            to delete.

        Possible Errors:
        CIM_ERR_ACCESS_DENIED
        CIM_ERR_NOT_SUPPORTED
        CIM_ERR_INVALID_NAMESPACE
        CIM_ERR_INVALID_PARAMETER (including missing, duplicate, unrecognized
            or otherwise incorrect parameters)
        CIM_ERR_INVALID_CLASS (the CIM Class does not exist in the specified
            namespace)
        CIM_ERR_NOT_FOUND (the CIM Class does exist, but the requested CIM
            Instance does not exist in the specified namespace)
        CIM_ERR_FAILED (some other unspecified error occurred)
        """
        raise pywbem.CIMError(pywbem.CIM_ERR_NOT_SUPPORTED)

    @cmpi_logging.trace_method
    def references(self, env, object_name, model, result_class_name, role,
                   result_role, keys_only):
        """Instrument Associations.

        All four association-related operations (Associators, AssociatorNames,
        References, ReferenceNames) are mapped to this method.
        This method is a python generator

        Keyword arguments:
        env -- Provider Environment (pycimmb.ProviderEnvironment)
        object_name -- A pywbem.CIMInstanceName that defines the source
            CIM Object whose associated Objects are to be returned.
        model -- A template pywbem.CIMInstance to serve as a model
            of the objects to be returned.  Only properties present on this
            model need to be set.
        result_class_name -- If not empty, this string acts as a filter on
            the returned set of Instances by mandating that each returned
            Instances MUST represent an association between object_name
            and an Instance of a Class whose name matches this parameter
            or a subclass.
        role -- If not empty, MUST be a valid Property name. It acts as a
            filter on the returned set of Instances by mandating that each
            returned Instance MUST refer to object_name via a Property
            whose name matches the value of this parameter.
        result_role -- If not empty, MUST be a valid Property name. It acts
            as a filter on the returned set of Instances by mandating that
            each returned Instance MUST represent associations of
            object_name to other Instances, where the other Instances play
            the specified result_role in the association (i.e. the
            name of the Property in the Association Class that refers to
            the Object related to object_name MUST match the value of this
            parameter).
        keys_only -- A boolean.  True if only the key properties should be
            set on the generated instances.

        The following diagram may be helpful in understanding the role,
        result_role, and result_class_name parameters.
        +------------------------+                    +-------------------+
        | object_name.classname  |                    | result_class_name |
        | ~~~~~~~~~~~~~~~~~~~~~  |                    | ~~~~~~~~~~~~~~~~~ |
        +------------------------+                    +-------------------+
           |              +-----------------------------------+      |
           |              |  [Association] model.classname    |      |
           | object_name  |  ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~    |      |
           +--------------+ object_name.classname REF role    |      |
        (CIMInstanceName) | result_class_name REF result_role +------+
                          |                                   |(CIMInstanceName)
                          +-----------------------------------+

        Possible Errors:
        CIM_ERR_ACCESS_DENIED
        CIM_ERR_NOT_SUPPORTED
        CIM_ERR_INVALID_NAMESPACE
        CIM_ERR_INVALID_PARAMETER (including missing, duplicate, unrecognized
            or otherwise incorrect parameters)
        CIM_ERR_FAILED (some other unspecified error occurred)
        """
        handlers = [
                ("Job", "LMI_SoftwareJob", generate_job_referents),
                ("JobParameters", "LMI_SoftwareMethodResult",
                    generate_result_referents)
        ]

        for ref in generate_references(env, object_name, model,
                result_class_name, role, result_role, keys_only, handlers):
            yield ref




# -*- encoding: utf-8 -*-
# Software Management Providers
#
# Copyright (C) 2012-2013 Red Hat, Inc.  All rights reserved.
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
#
# Authors: Michal Minar <miminar@redhat.com>
#
"""
Exceptions raisable by YumWorker.
"""

class YumDBError(Exception):
    """Base class for all errors under yumdb package."""

class DatabaseLockError(YumDBError):
    """Raised, when the yum database can not be locked."""
    pass

class TransactionError(YumDBError):
    """Base exception representing yum transaction processing error."""
    pass
class TransactionBuildFailed(TransactionError):
    """Raised, when transaction building fails."""
    pass
class PackageAlreadyInstalled(TransactionError):
    """Raised, when trying to install already installed package."""
    def __init__(self, pkg):
        TransactionError.__init__(self,
                'Package "%s" is already installed.' % pkg)
class PackageOpenError(TransactionError):
    """Raised, when trying to open package obtained from URI."""
    def __init__(self, pkg, msg):
        TransactionError.__init__(self,
                'Failed to open package "%s": %s' % (pkg, msg))
class TransactionExecutionFailed(TransactionError):
    """Raised, when YumBase.doTransaction() method fails."""
    pass

class TerminatingError(TransactionError):
    """Raised when job can not be completed due to provider's termination."""
    pass

class PackageError(YumDBError):
    """Generic exception for error concerning package handling."""
    pass
class PackageNotFound(PackageError):
    """Raised, when requested package could not be found."""
    pass
class PackageNotInstalled(PackageError):
    """Raised, when requested package is not installed for desired action."""
    def __init__(self, pkg):
        PackageError.__init__(self, 'Package "%s" is not installed.' % pkg)
class FileNotFound(PackageError):
    """
    Raised, when requesting check on file that does not belong to
    particular package.
    """
    pass

class RepositoryError(YumDBError):
    """Generic exception for error concerning repository handling."""
    pass
class RepositoryNotFound(RepositoryError):
    """Raised, when requested repository cound not be found."""
    def __init__(self, repoid):
        RepositoryError.__init__(self, "No such repository: %s" % repoid)
class RepositoryChangeError(RepositoryError):
    """Raised, when modification of repository failed."""
    pass

class JobError(YumDBError):
    """Generic exception for job handling."""
    pass
class UnknownJob(JobError):
    """Raised, when no handler is available for given job on worker."""
    pass
class InvalidURI(JobError):
    """Raised, when passed uri is not a valid one."""
    def __init__(self, uri):
        JobError.__init__(self, "Invalid uri: \"%s\"" % uri)
class InvalidNevra(JobError):
    """Raised when trying to instantiate job with invalid nevra string."""
    pass
class JobControlError(JobError):
    """Generic exception for management of asynchronous jobs."""
    pass
class JobNotFound(JobControlError):
    """Raised upon request for not existing asynchronous job."""
    def __init__(self, target):
        JobControlError.__init__(self, "job %s could not be found" % target)
class InvalidJobState(JobControlError):
    """
    Raised when requested operation can not be executed on job in
    its current state.
    """
    pass

class IndicationError(YumDBError):
    """Generic error for indication handling."""
    pass




# -*- encoding: utf-8 -*-
# Software Management Providers
#
# Copyright (C) 2012-2013 Red Hat, Inc.  All rights reserved.
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA

"""
Just a common functionality related to class
LMI_SoftwareInstallationServiceAffectsElement.
"""

import pywbem

from lmi.providers import cmpi_logging
from lmi.providers import ComputerSystem
from lmi.software.core import Identity

class Values(object):
    class ElementEffects(object):
        Unknown = pywbem.Uint16(0)
        Other = pywbem.Uint16(1)
        Exclusive_Use = pywbem.Uint16(2)
        Performance_Impact = pywbem.Uint16(3)
        Element_Integrity = pywbem.Uint16(4)
        Manages = pywbem.Uint16(5)
        Consumes = pywbem.Uint16(6)
        Enhances_Integrity = pywbem.Uint16(7)
        Degrades_Integrity = pywbem.Uint16(8)
        Enhances_Performance = pywbem.Uint16(9)
        Degrades_Performance = pywbem.Uint16(10)
        # DMTF_Reserved = ..
        # Vendor_Reserved = 0x8000..0xFFFF
        _reverse_map = {
                0  : 'Unknown',
                1  : 'Other',
                2  : 'Exclusive Use',
                3  : 'Performance Impact',
                4  : 'Element Integrity',
                5  : 'Manages',
                6  : 'Consumes',
                7  : 'Enhances Integrity',
                8  : 'Degrades Integrity',
                9  : 'Enhances Performance',
                10 : 'Degrades Performance'
        }

@cmpi_logging.trace_function
def fill_model_computer_system(env, model, keys_only=True):
    """
    Fills model's AffectedElement and all non-key properties.
    """
    model["AffectedElement"] = ComputerSystem.get_path(env)
    if not keys_only:
        model["ElementEffects"] = [
                Values.ElementEffects.Enhances_Integrity,
                Values.ElementEffects.Degrades_Integrity,
                Values.ElementEffects.Other,
                Values.ElementEffects.Other]
        model["OtherElementEffectsDescriptions"] = [
                "Enhances Integrity",
                "Degrades Integrity",
                "Enhances Functionality",
                "Degrades Functionality"]
    return model

@cmpi_logging.trace_function
def fill_model_identity(model, pkg, keys_only=True, identity_model=None):
    """
    Fills model's AffectedElement and all non-key properties.
    """
    model["AffectedElement"] = Identity.pkg2model(pkg, model=identity_model)
    if not keys_only:
        model["ElementEffects"] = [Values.ElementEffects.Other]
        model["OtherElementEffectsDescriptions"] = ["Allows to install"]
    return model



from setuptools import setup
from lmi.pcp import __version__

setup(
    name='openlmi-pcp',
    description='PCP metric providers',
    author='Frank Ch. Eigler',
    author_email='fche@redhat.com',
    url='https://fedorahosted.org/openlmi/',
    version=__version__,
    namespace_packages=['lmi'],
    packages=[
        'lmi',
        'lmi.pcp'],
    install_requires=['openlmi', 'pcp'],
    license="LGPLv2+",
    classifiers=[
        'License :: OSI Approved :: GNU Lesser General Public License v2 or later (LGPLv2+)',
        'Operating System :: POSIX :: Linux',
        'Topic :: System :: Systems Administration',
        ]
    )



#!/usr/bin/python
# -*- Coding:utf-8 -*-
#
# Copyright (C) 2012-2013 Red Hat, Inc.  All rights reserved.
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details. #
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
#
# Authors: Radek Novacek <rnovacek@redhat.com>
# Authors: Michal Minar <miminar@redhat.com>
# Authors: Jan Grec <jgrec@redhat.com>

"""
Common test utilities.
"""

import platform
import pywbem
import re
from subprocess import call, check_output

RE_NEVRA = re.compile(
    r'^(?P<name>[^\s]+)-(?P<evra>(?P<epoch>\d+):(?P<ver>[^-\s]+)'
    r'-(?P<rel>[^\s]+)\.(?P<arch>[^.\s]+))$')
RE_NEVRA_OPT_EPOCH = re.compile(
    r'^(?P<name>[^\s]+)-(?P<evra>((?P<epoch>\d+):)?(?P<ver>[^-\s]+)'
    r'-(?P<rel>[^\s]+)\.(?P<arch>[^.\s]+))$')
RE_ENVRA = re.compile(
    r'^(?P<epoch>\d+|\(none\)):(?P<name>[^\s]+)-(?P<ver>[^-\s]+)'
    r'-(?P<rel>[^\s]+)\.(?P<arch>[^.\s]+)$')
RE_REPO = re.compile(
        r'(?:^\*?)(?P<name>[^\s/]+\b)(?!\s+id)', re.MULTILINE | re.IGNORECASE)

DEV_NULL = open('/dev/null', 'w')

def make_nevra(name, epoch, ver, rel, arch, with_epoch='NOT_ZERO'):
    """
    @param with_epoch may be one of:
        "NOT_ZERO" - include epoch only if it's not zero
        "ALWAYS"   - include epoch always
        "NEVER"    - do not include epoch at all
    """
    estr = ''
    if with_epoch.lower() == "always":
        estr = str(epoch)
    elif with_epoch.lower() == "not_zero":
        if epoch and str(epoch).lower() not in {"0", "(none)"}:
            estr = str(epoch)
    if len(estr):
        estr += ":"
    return "%s-%s%s-%s.%s" % (name, estr, ver, rel, arch)

def make_evra(epoch, ver, rel, arch):
    """ @return evra string """
    if not epoch or str(epoch).lower() == "(none)":
        epoch = "0"
    return "%s:%s-%s.%s" % (epoch, ver, rel, arch)

def run_yum(*params, **kwargs):
    """
    Runs yum with params and returns its output
    It's here especially to allow pass a repolist argument, that
    specifies list of repositories, to run the command on.
    """
    cmd = ['yum'] + list(params)
    repolist = kwargs.get('repolist', None)
    if repolist is None:
        repolist = []
    if repolist:
        cmd += ['--disablerepo=*']
        cmd += ['--enablerepo='+r for r in repolist]
    return check_output(cmd)

def get_repo_list():
    """
    @return list of software repository names
    """
    repos_str = check_output(['yum', 'repolist', '-q'])
    return RE_REPO.findall(repos_str)

def get_system_architecture():
    """
    @return the system architecture name as seen by rpm
    """
    return check_output(['rpm', '-q', '--qf', '%{ARCH}\n', 'rpm'])

def verify_pkg(name):
    """
    @return True, if package is installed and passes rpm verification check
    """
    return call(["rpm", "--quiet", "-Va", name]) == 0


def is_config_file(pkg, file_path):
    """
    @return True, if file_path is a configuration file of package pkg.
    """
    cmd = ['rpm', '-qc', pkg.name]
    out = check_output(cmd)
    return file_path in set(out.splitlines())   #pylint: disable=E1103

def is_doc_file(pkg, file_path):
    """
    @return True, if file_path is a documentation file of package pkg.
    """
    cmd = ['rpm', '-qd', pkg.name]
    out = check_output(cmd)
    return file_path in set(out.splitlines())   #pylint: disable=E1103

def make_identity_path(pkg, newer=True):
    """
    Make instance name for LMI_SoftwareIdentity from Package.
    """
    return pywbem.CIMInstanceName(
            classname="LMI_SoftwareIdentity",
            namespace="root/cimv2",
            keybindings=pywbem.NocaseDict({
                "InstanceID" : "LMI:LMI_SoftwareIdentity:%s" % pkg.get_nevra(
                    newer=newer)
            }))

def get_target_operating_system():
    """
    :returns: integer corresponding for TargetOperatingProperty of CIM_Check
        for this system
    """
    target_operating_system = 36 # LINUX
    if hasattr(platform, 'linux_distribution') and \
            platform.linux_distribution(
                    full_distribution_name=False)[0].lower() == 'redhat':
        target_operating_system = 79 # RHEL
        if platform.uname()[4].lower() == 'x86_64':
            target_operating_system = 80 # RHEL 64bit
    return pywbem.Uint16(target_operating_system)

def get_installed_packages():
    """
    :returns: list of packages in format: ``NAME-EPOCH:VERSION-RELEASE.ARCH``
    """
    output = check_output(
            [ "/usr/bin/rpm", "-qa", "--qf"
            , "%{NAME}-%{EPOCH}:%{VERSION}-%{RELEASE}.%{ARCH}\n"],
            stderr=DEV_NULL).split()
    package_list = []
    for package in output:
        # Skip all gpg public keys returned by rpm -qa
        if "gpg-pubkey" not in package:
            package_list.append(package.replace("(none)", "0"))
    return package_list

def make_pkg_op(ns, pkg):
    """
    :returns: Object path of ``LMI_SoftwareIdentity``
    :rtype: :py:class:`lmi.shell.LMIInstanceName`
    """
    if not isinstance(pkg, basestring):
        nevra = pkg.nevra
    else:
        nevra = pkg
    return ns.LMI_SoftwareIdentity.new_instance_name({
        "InstanceID" : 'LMI:LMI_SoftwareIdentity:' + nevra
    })





# -*- encoding: utf-8 -*-
# Software Management Providers
#
# Copyright (C) 2012-2013 Red Hat, Inc.  All rights reserved.
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA

"""
CIM values for enumeration types of CIM_InstMethodCall indication class.
"""

import pywbem

from lmi.providers import cmpi_logging
from lmi.software import util
from lmi.software.core import Job
from lmi.software.yumdb import jobs

class Values(object):
    class ReturnValueType(object):
        boolean = pywbem.Uint16(2)
        string = pywbem.Uint16(3)
        char16 = pywbem.Uint16(4)
        uint8 = pywbem.Uint16(5)
        sint8 = pywbem.Uint16(6)
        uint16 = pywbem.Uint16(7)
        sint16 = pywbem.Uint16(8)
        uint32 = pywbem.Uint16(9)
        sint32 = pywbem.Uint16(10)
        uint64 = pywbem.Uint16(11)
        sint64 = pywbem.Uint16(12)
        datetime = pywbem.Uint16(13)
        real32 = pywbem.Uint16(14)
        real64 = pywbem.Uint16(15)
        reference = pywbem.Uint16(16)
        # DMTF_Reserved = ..
        _reverse_map = {
                2: 'boolean',
                3: 'string',
                4: 'char16',
                5: 'uint8',
                6: 'sint8',
                7: 'uint16',
                8: 'sint16',
                9: 'uint32',
                10: 'sint32',
                11: 'uint64',
                12: 'sint64',
                13: 'datetime',
                14: 'real32',
                15: 'real64',
                16: 'reference'
        }

    class PerceivedSeverity(object):
        Unknown = pywbem.Uint16(0)
        Other = pywbem.Uint16(1)
        Information = pywbem.Uint16(2)
        Degraded_Warning = pywbem.Uint16(3)
        Minor = pywbem.Uint16(4)
        Major = pywbem.Uint16(5)
        Critical = pywbem.Uint16(6)
        Fatal_NonRecoverable = pywbem.Uint16(7)
        # DMTF_Reserved = ..
        _reverse_map = {
                0: 'Unknown',
                1: 'Other',
                2: 'Information',
                3: 'Degraded/Warning',
                4: 'Minor',
                5: 'Major',
                6: 'Critical',
                7: 'Fatal/NonRecoverable'
        }

@cmpi_logging.trace_function
def job2model(env, job, pre=True):
    """
    Create post or pre indication instance used by clients to subscribe
    to job's state changes.

    :param job: (``YumJob``) Instance of job created as a result of method
        invocation.
    :param pre: (``bool``) says, whether to make pre or post indication
        instance.
    :rtype CIMInstance of CIM_InstMethodCall.
    """
    if not isinstance(job, jobs.YumJob):
        raise TypeError("job must be a YumJob")
    if not pre and job.state == job.NEW or job.state == job.RUNNING:
        raise ValueError("job must be finished to make a post indication"
            " instance")
    path = util.new_instance_name("CIM_InstMethodCall")
    inst = pywbem.CIMInstance(classname="CIM_InstMethodCall", path=path)
    src_instance = Job.job2model(job, keys_only=False)
    inst['SourceInstance'] = pywbem.CIMProperty("SourceInstance",
            type="instance", value=src_instance)
    inst['SourceInstanceModelPath'] = \
            str(src_instance.path)    #pylint: disable=E1103
    method_name = Job.JOB_METHOD_NAMES[job.metadata["method"]]
    inst['MethodName'] = method_name
    # TODO: uncomment when Pegasus can correctly handle instances
    # of unregistered classes
    #inst['MethodParameters'] = Job.make_method_params(
    #        job, '__MethodParameters', True, not pre)
    # TODO: until then, use this workaround
    if not pre:
        inst["MethodParameters"] = Job.make_method_params(
                job, '__MethodParameters_' + method_name, False, True)

    inst['PreCall'] = pre

    if not pre:
        inst["Error"] = pywbem.CIMProperty("Error", type="instance",
                is_array=True, value=[])
        error = Job.job2error(env, job)
        if error is not None:
            inst["Error"].append(error)
        inst["ReturnValueType"] = Values.ReturnValueType.uint32
        return_value = Job.make_return_value(job)
        if return_value is not None:
            inst["ReturnValue"] = str(return_value)

    return inst

